/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.impactanalysis.rmps.host.web                            |
 *|                                                                        |
 *| Copyright IBM Corporation 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function () {
	//
	//	Implement DM diagram utility methods for Impact Analysis
	//

	dojo.provide("impactanalysis.app.rmps.DMDiagramUtilityMethods");

	dojo.require("dm.analysis.app.ui.DiagramUtilityMethods");

	dojo.require("rmps.app.core.util");
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.util.ContextUtils");
	dojo.require("rmps.core.util.IRmpsConstants");
	
	var REGISTERED_DM_URI = "http://jazz.net/ns/dm/",
		_isDMResource,
		getCompactRenderingUri,
		getResourceEditUri;
	
	/** Determine if a resource URI corresponds to a DM resource 
	 *  which can be rendered and edited. URIs that start with the
	 *  DM front-side URL or URIs that belong to the registered
	 *  DM name space on Jazz.net can be rendered by DM.
	 */
	_isDMResource = function (resourceUri) {
		return (resourceUri.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) === 0 ||
				resourceUri.indexOf(REGISTERED_DM_URI) === 0);
	};

	/** Compute the URI to perform the compact rendering
	 *  of the corresponding resource. If the resource is
	 *  on DM, use the root services "rendering" URL.
	 *  Otherwise, use the resource URI itself.
	 */
	getCompactRenderingUri = function (resourceUri) {
		// If the resource is not from our server, use the resource URI as the rendering URI
		var renderingUri = resourceUri;
		if (_isDMResource(resourceUri)) {
			// Compute the URI for rendering the resource information
			var object= {
				resourceUri: resourceUri
			};
			var contextUri = rmps.core.util.ContextUtils.getCurrentContextURI();
			if (contextUri) {
				object[rmps.core.util.IRmpsConstants.BASELINE]= contextUri;
			}
			renderingUri = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rendering") + "?" + dojo.objectToQuery(object);
		}
		
		return renderingUri;
	};


	/** Compute the URI to open the editor of 
	 *  the corresponding resource. If the resource is
	 *  on DM, use design manager editor.
	 *  Otherwise, use the resource URI itself.
	 */
	getResourceEditUri = function (resourceUri) {
		// If the resource is not from our server, use the resource URI as the editor URI
		var editorUri = resourceUri;
		if (_isDMResource(resourceUri)) {
			// Compute the URI for opening the resource editor
			editorUri = rmps.app.core.util.getDesignViewResourceUri(resourceUri);
		}
		return editorUri;
	};

	// Register the utility methods with the common IA
	dm.analysis.app.ui.DiagramUtilityMethods.set_getCompactRenderingUri(getCompactRenderingUri);
	dm.analysis.app.ui.DiagramUtilityMethods.set_getResourceEditUri(getResourceEditUri);

}());
