/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.impactanalysis.rmps.host.web                            |
 *|                                                                        |
 *| Copyright IBM Corporation 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function () {
	// summary:
	//	Implement a resource picker for Impact Analysis
	//
	// description:
	//	Uses RMPSPicker (simple search on DM repository) as a picker.
	//	Note: This module is loaded by the com.ibm.xtools.rmps.web.app.webUiInitializers extension point
	//	so that it can install the picker functionality for IA. See com.ibm.xtools.impactanalysis.rmps.host.web/plugin.xml

	dojo.provide('impactanalysis.app.rmps.SimpleIAResourceProvider');

	dojo.require("dojo.i18n");

	dojo.require('dm.analysis.app.ui.picker');
	dojo.require('jazz.app.context');
	dojo.require('rmps.app.pickers.internal.RMPSPicker');

	dojo.requireLocalization("impactanalysis.app.rmps", "Messages");

	var messages = dojo.i18n.getLocalization("impactanalysis.app.rmps", "Messages"),
		pickerFunction;

	pickerFunction = function (callbackFunction) {
		// summary:
		//		the function that opens the RMPS picker for IA
		// description:
		//		opens a DM resource picker (RMPSPicker), and when a resource is selected
		//		informs IA by calling the passed callback function with the resource URI
		var picker = null,
			onCloseCallback,
			content,
			floatingPaneParams;

		onCloseCallback = dojo.hitch(this, function (results) {
			if (picker) {
				picker.close();
				picker.destroyRecursive();
			}
			if(results && results[0]) {
				callbackFunction(results[0].link);
			}
		});

		content = new rmps.app.pickers.internal.RMPSPicker({
			projectId: jazz.app.context.get().itemId,
			context: jazz.app.context.get().baseline,
			populateOSLCResponse: false,
			filteredList: [], // put added elements here 
			findFragments: true,
			onClose: onCloseCallback
		});

		floatingPaneParams = {
            primaryTitle: messages.pickerPrimaryTitle,
            secondaryTitle: messages.pickerSecondaryTitle,
            width: "800px",
            contentNode: content.domNode
        };
        picker = jazz.ui.Dialog(floatingPaneParams);
	};

	// register the picker function with IA
	dm.analysis.app.ui.picker.setPicker(pickerFunction);

}());
