/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	dojo.provide('dm.analysis.app.ui.internal.diagram.DiagramLoader');

	dojo.require('dojo.i18n');

	dojo.require('dojox.lang.functional');
	
	dojo.require('dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer');
	dojo.require('dm.analysis.app.ui.internal.diagram.TimeSlicedIterator');
	
	dojo.requireLocalization('dm.analysis.app.ui.internal.impactanalysis', 'ImpactAnalysisMessages'); 
	
	var df,
		messages;
	
	df = dojox.lang.functional;
	messages = dojo.i18n.getLocalization('dm.analysis.app.ui.internal.impactanalysis', 'ImpactAnalysisMessages');
	
	dojo.declare('dm.analysis.app.ui.internal.diagram.DiagramLoader', [], {
		// summary:
		//		Perform asynchronous graph loading for impact analysis.
		// description:
		//		Extracts portions of ibm_ilog.diagram.widget.DataConnector (the portion of iLog
		//		that loads a graph from a data store), and makes the loading run
		//		'in the background'.  
		//		Methods copied/hacked/modified from DataConnector all begin with _djd_.
	
		_diagram: null,
		_resourceStore: null,
		_linkStore: null,
	
		constructor: function (diagram, resourceStore, linkStore) {
			this._diagram = diagram;
			this._resourceStore = resourceStore;
			this._linkStore = linkStore;
			
		},
		
		_createLoadSequencer: function () {
			// summary:
			//		Defines the loading sequence for a diagram as a series of steps.
			//	description
			//		Much of this is extracted from DataConnector._load
			var sequencer;
			
			sequencer = new dm.analysis.app.ui.internal.diagram.LongRunningWorkSequencer()
			
		//		.step(messages.diagramLoadInit, dojo.hitch(this, function() {
		// N.B. At least in FF, hiding the surface slows things down !?!
		//				this._diagram.getSurface().rawNode.style.display = 'none';
		//		}))
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
					// summary:
					//		Hack in our a private version on this._diagram._clearNodes 
					this._diagram._clearNodes = function() {
						// summary:
						//		Clear the current collection of nodes created from the nodes store.
						// tags:
						//		private
						// clear graphical shapes
						this._model.forEachEntry(function(e) {
							// remove all links attached to this node (both linksStore and implicits)
							// START_DM_CHANGE
							var links = e.node ? e.node.getLinks() : [];
							// END DM_CHANGE
							dojo.forEach(links, function(link) {
								this._linkDispose(link);
							}, this);
							// and dispose the node
							this._nodeDispose(e.node);
						}, this);
						// clear data model
						this._model.initNodes();
						// also the links since all the links shapes have been removed
						this._model.initLinks();
					};
				}))
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
					// summary:
					//		Loads the 'node model' of the diagram. Isn't slow, but refactored
					//		this into a time sliced looper anyhow.
					var stepIncomplete;
					
					s.iterator = s.iterator 
						|| new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('nodes', data.resources);
						
					s.iterator.forWorkIncrement(this._diagram._addStoreItemToModel, this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function(s, data) {
					// summary:
					//		Loads the 'link model' of the diagram. Isn't slow, but refactored
					//		this into a time sliced looper anyhow.
					var stepIncomplete;
					
					s.iterator = s.iterator 
						|| new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('links', data.links);
						
					s.iterator.forWorkIncrement(this._djd_addLinkStoreItemToModel, this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateModel, dojo.hitch(this, function () {
					// summary:
					//		Complete model construction. Is not typically slow.
					dojo.hitch(this._diagram, this._djd_resolveConnectionsFromNodesStore)();
					dojo.hitch(this._diagram, this._djd_completeModelConstruction)();
				}))
				.step(messages.diagramLoadCreateShapes, dojo.hitch(this, function (s) {
					// summary:
					//		Incrementally create model shapes. Can be terribly slow, esp. in IE.
					var stepIncomplete;
					
					s.iterator = s.iterator || this._djd_init_shapeCreationIncrementalInterator();

					s.iterator.forWorkIncrement(
						this._djd_get_createShapesFromModel_workIncrement(),
						this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateShapes, dojo.hitch(this, function (s) {
					// summary:
					//		Incrementally create model links defined directly in nodes. 
					//		Can be terribly slow, esp. in IE, but in IA, is not because links are defined
					//		via a separate links store
					var stepIncomplete;
					
					s.iterator = s.iterator || this._djd_init_modelEntryInterator();

					s.iterator.forWorkIncrement(
						this._djd_createLinksFromModelEntry,
						this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadCreateLinks, dojo.hitch(this, function (s) {
					// summary:
					//		Incrementally create model links in the links store. 
					//		Can be terribly slow, esp. in IE.
					var stepIncomplete;
					
					s.iterator = s.iterator || this._djd_init_linkEntryIterator();

					s.iterator.forWorkIncrement(
						this._djd_createLinksFromModelLinkEntry,
						this._diagram);
					
					stepIncomplete = s.iterator.hasMore();

					return stepIncomplete;
				}))
				.step(messages.diagramLoadLayout, function () { 
					// do nothing, just give UI time to refresh before beginning layout
					return;
				})
				.step(messages.diagramLoadLayout, dojo.hitch(this, function () {
					// summary:
					//		Signal layout completion. In IA, this triggers graph layout, which can be slow,
					//		but should be handled separately...
					dojo.hitch(this._diagram, 'onLoaded')();
				}))
	//			.step(messages.diagramLoadLayout, dojo.hitch(this, function () {
	// N.B. At least in FF, hiding the surface slows things down !?!
	//					this._diagram.getSurface().rawNode.style.display = '';
	//			}))
				;
			
			return sequencer;
		},

		createNodesAndLinks: function () {
			// summary:
			//		Asynchronously create diagram nodes and links from the node and link stores.
			// description:
			//		Asynchronously create diagram nodes and links from the node and link stores for the diagram.
			//		Returns a dojo.Deferred with with the progress and result of the process can be monitored:
			//			loader.createNodesAndLinks().then( 
			//				function () { /*handle completion*/ {,
			//				function (error) { /* handle error */ },
			//				function (progress) { /* handle progress */ });
			//		The object passed to the progress function has the following fields:
			//			message: a localized message suitable for user display
			var graphData, sequencer;
			
			// Code stolen from ibm_ilog.diagram.widget.DataConnector._load()
			if (this._diagram._isValidModel()) {
				this._diagram._reset();
				// end stolen code

				graphData = { };
				sequencer = this._createLoadSequencer();
				
				this._resourceStore.fetch({onComplete: dojo.hitch(this, function (items) {
					graphData.resources = items;
					
					this._linkStore.fetch({onComplete: dojo.hitch(this, function (items) {
						graphData.links = items;
						
						sequencer.schedule(graphData);
					})});
				})});
			}
			return sequencer.getPromise();
		},
		
		_djd_resolveConnectionsFromNodesStore: function () {
			// summary:
			//		modified version of _resolveConnectionsFromNodesStore that just does resolution, and not next step processing
			this._model.forEachUnresolvedEntry(function(entry) {
			
				var currentItem = entry.item;
				var connectingItems;
				
				if (this._parentBindingFunction) {
					connectingItems = this._parentBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						
						if (this._model.containsEntry(ident)) {
							entry.parentlist.push(ident);
						}
					}, this);
				}
				
				if (this._childBindingFunction) {
					connectingItems = this._childBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.childlist.push(ident);
							
						}
					}, this);
				}
				
				if (this._successorsBindingFunction) {
					connectingItems = this._successorsBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.tolist.push(ident);
						}
					}, this);
				}
				
				if (this._predecessorsBindingFunction) {
					connectingItems = this._predecessorsBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.fromlist.push(ident);
						}
					}, this);
				}
				
				entry.resolved = true;
			}, this);
			
			this._model.setAllEntriesAdded();
			
		},
		
		_djd_completeModelConstruction: function () {
			// summary:
			//		Work incrementalized version of _completeModelConstruction -- does not create shapes
			
			if(!this._model.isModelComplete()) {
				console.error('_djd_completeModelConstruction: model is not complete');
			}
			// When all model entries have been creates and their items loaded..            
			if (this._model.isModelComplete()) {
				//                
				// Assumptions made for hierarchies
				// - An entry with an empty to-list is a leaf node 
				// - The from-list length will by not longer than 1
				
				this._model.forEachEntry(function(entry) {
					var childlist = entry.childlist;
					var parentlist = entry.parentlist;
					if (childlist.length > 0) {
						var t;
						for ( t = 0; t < childlist.length; t++) {
							var childEntry = this._model.getEntry(childlist[t]);
							childEntry.parent = entry;
							entry.children.push(childEntry);
						}
					}
					if (parentlist.length === 1) {
						var parentEntry = this._model.getEntry(parentlist[0]);
						entry.parent = parentEntry;
						parentEntry.children.push(entry);
					}
				}, this);
				
				this._onModelCreated(this._model);
			}
			
		},
		
		_djd_addLinkStoreItemToModel: function (item) {
			// code copied from ibm_ilog.diagram.widget.DataConnector._resolveConnectionsFromLinksStore
			var linkEntry = this._addLinkStoreItemToModel(item);
			linkEntry.startId = this._getStartLinkItemId(item);
			linkEntry.endId = this._getEndLinkItemId(item);
			
			// If we have enough information to describe a link then
			// update the model. 
			if (linkEntry.startId !== null) {
				linkEntry.start = this._model.getEntry(linkEntry.startId);
			}
			if (linkEntry.endId !== null) {
				linkEntry.end = this._model.getEntry(linkEntry.endId);
			}
			
			if (!linkEntry.start || !linkEntry.end) {
				linkEntry.pending = true;
			}
		},
		
		_djd_init_shapeCreationIncrementalInterator: function () {
			var iter,
				d = this._diagram;
			if (d._createLinksForHierarchy) {
				iter = this._djd_init_modelEntryInterator();
			} else {
				iter = new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('subgraphs', 
					df.filter(d._model.nmodel.getValueList(), function (v) { return !v.parent; })
				);
			}
			return iter;
		},
		
		_djd_get_createShapesFromModel_workIncrement: function() {
			// summary:
			//		return a work increment for creating a shape
			
			var workFunction;
			
			if (this._diagram._createLinksForHierarchy) {
				workFunction = function (entry) {
					// Create nodes
					entry.node = this.createNode(null, entry.item);
					this._applyNodeStoreItemGraphics(entry.node);
				};
			} else {
				workFunction = function (entry) {
					// Create subgraphs/nodes
					if (entry.isLeaf()) {
						entry.node = this.createNode(null, entry.item);
						this._applyNodeStoreItemGraphics(entry.node);
					} else {
						entry.node = this.createSubgraph(null, entry.item);
						this.createChildGraphics(entry);
						this._applyNodeStoreItemGraphics(entry.node);
					}
				};
			}
			return workFunction;
		},
		
		_djd_createShapesFromModel_workIncrement: function(iter) {
			// summary:
			//		Incrementalized version of _createShapesFromModel
			if (this._createLinksForHierarchy) {
				// Create nodes
				iter.forWorkIncrement(function(entry) {
					entry.node = this.createNode(null, entry.item);
					this._applyNodeStoreItemGraphics(entry.node);
				}, this);
			} else {
				// Create subgraphs/nodes
				iter.forWorkIncrement(function(entry) {
					if (entry.isLeaf()) {
						entry.node = this.createNode(null, entry.item);
						this._applyNodeStoreItemGraphics(entry.node);
					} else {
						entry.node = this.createSubgraph(null, entry.item);
						this.createChildGraphics(entry);
						this._applyNodeStoreItemGraphics(entry.node);
					}
				}, this);
			}
		
		},
		
		_djd_init_modelEntryInterator: function () {
			return new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('nodes', this._diagram._model.nmodel.getValueList());
		},
		
		_djd_init_linkEntryIterator: function () {
			return new dm.analysis.app.ui.internal.diagram.TimeSlicedIterator('links', this._diagram._model.lmodel.getValueList());
		},

		_djd_createLinksFromModelEntry: function (entry) {
			// summary:
			//		Functionality extracted from DataConnector._createLinksFromModel
			var currentNode;
			var childlist = entry.childlist;
			var parentlist = entry.parentlist;
			var tolist = entry.tolist;
			var fromlist = entry.fromlist;
			var link, t, f;
			if (this._createLinksForHierarchy) {
			
				if (childlist.length + parentlist.length > 0) {
					currentNode = entry.node;
					for (t = 0; t < childlist.length; t++) {
						var childNode = this._model.getEntry(childlist[t]).node;
						link = this.connectNodes(currentNode, childNode);
					}
					for (f = 0; f < parentlist.length; f++) {
						var parentNode = this._model.getEntry(parentlist[f]).node;
						link = this.connectNodes(parentNode, currentNode);
					}
				}
			}
			if (tolist.length + fromlist.length > 0) {
				currentNode = entry.node;
				for (t = 0; t < tolist.length; t++) {
					var toNode = this._model.getEntry(tolist[t]).node;
					link = this.connectNodes(currentNode, toNode);
				}
				for (f = 0; f < fromlist.length; f++) {
					var fromNode = this._model.getEntry(fromlist[f]).node;
					link = this.connectNodes(fromNode, currentNode);
				}
			}
		},
		
		_djd_createLinksFromModelLinkEntry: function (entry) {
			// summary:
			//		Functionality extracted from DataConnector._createLinksFromModel
			if (!entry.pending) {
				entry.link = this.connectNodes(entry.start.node, entry.end.node, null, entry.item);
				this._applyLinkStoreItemGraphics(entry.link);
			}
		}
		
		
	});
	
}());