/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram");

dojo.require("dojo.i18n");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.string");

dojo.require('dojox.html.entities');

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Select");
dojo.require("dijit.form.HorizontalSlider");

dojo.require("jazz.util.html");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.tree.IconItem");
dojo.require("jazz.ui.tree.SimpleItem");
dojo.require("rmps.ui.CollapsibleSection");

dojo.config.useGfxLayout = true;

dojo.require('dm.analysis.app.ui.internal.diagram.DiagramLoader');

// IMPORTANT. Do NOT require ibm_ilog.xxx resources directly. The Dojo Diagramer 'layer' files below
// include all required classes, and avoid ciruclar dependencies
dojo.require('ibm_ilog.graphlayout.ibm_graphlayout_base');
dojo.require('ibm_ilog.diagram.ibm_diagram_base');

dojo.require('ibm_ilog.graphlayout.ibm_graphlayout_basiclinkstyle');
dojo.require('ibm_ilog.graphlayout.ibm_graphlayout_tree');
dojo.require('ibm_ilog.graphlayout.ibm_graphlayout_hierarchical');
dojo.require('ibm_ilog.graphlayout.ibm_graphlayout_forcedirected');
dojo.require('ibm_ilog.graphlayout.ibm_graphlayout_circular');

 
dojo.require("dm.analysis.app.ui.internal.impactanalysis.AddImpactComentDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisCommon");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisTree");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.LegendDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.IAColorTable");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 
var common =  dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisCommon;
var colorTable = dm.analysis.app.ui.internal.impactanalysis.IAColorTable;

var minZoom = 0.1;
var maxZoom = 3;
var zoomFactor = 1.2;

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram", [dijit._Widget, dijit._Templated], {
	
	messages:messages,
	selectedItem:null,
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl('dm.analysis.app','ui/internal/impactanalysis/templates/ImpactAnalysisDiagram.html'),

	toolbarHeight: 35,
	TOOLBAR_HEIGHT: 35,
	
	// init properties
	editor: null,
	subgraphs:null,
	markSubgraph:null,
	treeSection:null,
	allElements:null, /* need to impactAnalysis tree */
	
	// Keep a handle to the pop-up menu
	_popupMenu: null,
	
	// remember the node that has the focus
	_focusNode: null,
	
	// remember the graph node corresponding to the primary target
	_targetNode: null,

	// for removed elements
	originalResult:null,
	currentMapRes:null,
	/*
	 * capture for each removed elements what are the associated elements, result
	 * {URI:{elements:[] , relations:[]}}
	 */
	removedElement:null,
	distance:null,
	slider:null,
	toolbarSlider:null,
	listenToSlider:true,
	mustUpdateSlider:true,
	zoomChanged:false,
	//
	toolbar:null,
	filterSelect:null,
	resetzoom:null,
	fitzoom:null,
	
	_getCurrentLayout: function () {
		// summary:
		//		Return the Dojo Diagrammer layout corresponding to the current selected layout

		var layoutType = this.filterSelect.value || common.LAYOUT_HIERARCHICAL;
		return common.selectLayout(layoutType);
	},

	postCreate: function() {
		this._loadToolButtons();
		
        ////////////////////////////////////// create the main diagram diagram ///////////////////////////////////////////////////////
		
		// Create an instance of the pop-up menu for node actions
		this._popupMenu = new dm.analysis.app.ui.internal.impactanalysis.AnalysisPopupMenu({
			editor: this.editor,
			diagram: this
		});
		
        this.impactDiagram.set('nodesQuery', {
			URI: '*'
        });
        this.impactDiagram.set('nodesQueryOptions', {
			deep: true
        });
        this.impactDiagram.set('childBinding', "nodeof");
        this.impactDiagram.set('createLinksForHierarchy', false);
        this.impactDiagram.set('nodeLayout', this._getCurrentLayout());
        this.impactDiagram.set('viewportResizeMode', 1);
        this.impactDiagram.set('automaticNodeLayout', false);
        this.impactDiagram.set('overview', true);
        this.impactDiagram.set('move', true);

        // Note: For Mobile browsers, marquee and mouse wheels are not implemented.
        // Trying to disable them on mobile browsers results in errors.
		if (!dojo.config.diagramForMobile) {
        	this.impactDiagram.set('marquee', false); //disable selection rectangle.
        	this.impactDiagram.set('wheelPan', false); //disable the vertical scrollbar.
        }
		dojo.style(this.impactDiagram.scrollbarsNode, 'overflow', 'hidden'); //hide the vertical and horizontal scrollbars

        //Change the input filter for the Pan Interactor's start event to only require a left click (and not the shift key):
        this.impactDiagram.assignInteractorInputs("pan", { start: { filter: { button:0 } }});
        
        //Change the input filter for the Zoom Interactor's wheelZoom event to no longer require any keypress before zooming:
        this.impactDiagram.assignInteractorInputs("zoom", { wheelZoom: { filter: {} }});

        this.impactDiagram.set('viewportResizeMode', 1);
        //this.impactDiagram.set ('showZoomFactor',true);
        this.impactDiagram.set('showZoomFeedback', true);
        this.impactDiagram.set('viewportAnimated', true);
        this.impactDiagram.set('nodeTemplate', ibm_ilog.diagram.declareTemplate(common.getTemplate()));
        //this.impactDiagram._setOverviewRendererAttr(common.getOverviewRenderer());
        this.connect(this.impactDiagram, "onNodeClick", this.onNodeClick);
        this.connect(this.impactDiagram, "onNodeCreated", this.onNodeCreated);
        this.connect(this.impactDiagram, "onLoaded", this.performLayout);
        this.connect(this.impactDiagram, "onLinkCreated", this.onLinkCreated);
        this.connect(this.impactDiagram, "onFocusedElementChanged", this.onFocusedElementChanged);
        this.connect(this.impactDiagram, "onKeyPress", this.onKeyPress);
        this.connect(this.impactDiagram, "onNodeMouseEnter", this.onNodeMouseEnter);
        this.connect(this.impactDiagram, "onNodeMouseLeave", this.onNodeMouseLeave);
        this.connect(this.impactDiagram, "onSubgraphCreated", this.setGridLayoutOnSubgraph);
		// zooming options
		var VP =  this.impactDiagram.getViewport();

		VP.setMaxZoom(maxZoom);
		VP.setMinZoom(minZoom);
		//VP.setZoom(0.8);
		//VP.setAirFactor(0.05);
		this.connect(VP, "onViewRectChanged", function(){ this.onViewportChange();});
		this.updateScrollbarsAndZoom();

		VP.setAnimated(true);
		this.impactDiagram.set ('overviewSize', {width: 160,height: 120 });
		VP.setHudEnabled(true);
		this.treeSection = this.editor.getTreeSideNav();
		this.resize(this.size);
		
		// for Chrome: doesn't autosize the 'canvasNode' div to its content (the SVG)
		// Explicitly style it to 100% width.
		this.impactDiagram.canvasNode.style.width = '100%';
		
	},

	_loadToolButtons: function()
	{
		var self = this;
		this.toolbar = new jazz.ui.SimpleToolbar({});
		dojo.style(this.toolbar, 'height', this.TOOLBAR_HEIGHT + 'px');
		dojo.style(this.toolbarNode, 'height', this.TOOLBAR_HEIGHT + 'px');
		
		this.toolbarNode.appendChild(this.toolbar.domNode);
		this.filterSelect = new dijit.form.Select({
			name: 'filterSelect',
			style: 'width:150px;',
			options: [{
				label: messages.layoutHierarchical,
				value: common.LAYOUT_HIERARCHICAL,
				selected: true
			},
			{
				label: messages.layoutTree,
				value: common.LAYOUT_TREE,
				selected: false
			},
			{
				label: messages.layoutRadialTree,
				value: common.LAYOUT_RADIAL_TREE,
				selected: false
			},
			{
				label: messages.layoutForceDirected,
				value: common.LAYOUT_FORCE_DIRECTED,
				selected: false
			},
			{
				label: messages.layoutCircular,
				value: common.LAYOUT_CIRCULAR,
				selected: false
			}]
		});
		this.connect(this.filterSelect, "onChange", function(){ 
			this.setLayout();
		});
	
		this.toolbar.addWidget(this.filterSelect);
		this.toolbar.addSeparator();
		
		this.toolbarSlider = new dijit.form.HorizontalSlider({
			name: "slider",
			value: 80,
			minimum: 5,
			maximum: 200,
			intermediateChanges: true,
			style: "width:200px;",
		//	showButtons: false,
			onChange: function(value) {
				self.scalingEvent(value);
			}

		},
		"slider");
		this.toolbar.addWidget(this.toolbarSlider);
		
		this.resetzoom = this.toolbar.addButton(this, this.resetZoom, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/resetzoom.png'),
			label:  messages.resetZoom
		});

		this.fitzoom = this.toolbar.addButton(this, this.fitZoom, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/zoomfit.png'),
			label: messages.zoomFit
		});
		
		this.toolbar.addSeparator();
		
		this.openLegend = this.toolbar.addButton(this, this.openLegendDlg, {
			imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/legend.gif'),
			label: messages.openLegend
		});
    },	

	startup: function(){
		this.inherited(arguments);
	},
	
	//////// layouting
	setLayout: function(){
		var layoutType = this.filterSelect.value,
			i;
		this.impactDiagram.set("nodeLayout",  common.selectLayout(layoutType));
		this.impactDiagram.performGraphLayout();
		for (i = 0 ; i < this.subgraphs.length ; i++)
		{
			this.subgraphs[i].setCollapsed(true);
		}
	},
	
	
	/////// handle zooming functions
	
	
	scalingEvent : function(value){
		var V = this.impactDiagram.getViewport();
	//	var value = this.slider.value;
	    var zoom = this.sliderToZoom(value);
	    this.mustUpdateSlider = false;
	    if (this.listenToSlider && zoom != V.getZoom()) {
	        this.setZoom(zoom);
	    }
	    this.mustUpdateSlider = true;
	},
	
	updateScrollbarsAndZoom : function(){
	    this.updateButtons();
	    this.updateSlider();
	},

	// Update the UI when the zoom is changed
	onViewportChange : function(){
		this.updateScrollbarsAndZoom();
	},

	setZoom : function(zoom){
		var V = this.impactDiagram.getViewport();
		if (zoom && zoom>=V.getMinZoom() && zoom<=V.getMaxZoom()) {
	        V.setZoom(zoom, {
	            instant: true
	        });
	    }
	},
	
	_focusOnTarget: function() {
		setTimeout(dojo.hitch(this, function () { this.impactDiagram.centerOnNode(this._targetNode); }), 100);
	},

	sliderToZoom : function(value){
		return value / 100;
	},

	zoomToSlider : function(zoom){
		return zoom * 100;
	},

	updateSlider : function(){
		if (this.mustUpdateSlider == true) {
			var zoom = this.impactDiagram.getViewport().getZoom();
			this.listenToSlider = false;
			this.toolbarSlider.set('value', this.zoomToSlider(zoom));
		    this.listenToSlider = true;
		}
	},

	updateButtons : function(){
	    var currentZoom = this.impactDiagram.getViewport().getZoom();
	},

	resetZoom : function(){
		this.setZoom(1);
		this._focusOnTarget();
	},

	fitZoom : function(){
		var vp = this.impactDiagram.getViewport();
		vp.setViewRect(vp.getContentRect(), {instant: true});
	},
	
	openLegendDlg : function()
	{
		if (this.originalResult){
			var legendDlg = new dm.analysis.app.ui.internal.impactanalysis.LegendDialog(this,this.originalResult.toolsData);
		}
	},
	
	/// finish zooming functions

	resize: function(size)
	{
		var dn = this.impactDiagram.domNode,
			h;
			
		if (size.h === undefined) {
			h = dojo.contentBox(dn).h;
		} else {
			// diagram widget is expecting a CONTENT height. 
			// adjust by removing tb height + difference between margin & content box heights
			// of the widget (i.e. the height of borders, padding, etc...
			h = size.h - this.TOOLBAR_HEIGHT 
				- (dojo.marginBox(dn).h - dojo.contentBox(dn).h);
		}
		this.impactDiagram.resize({ w: size.w, h: h });
	},

	reset: function(){
		this.impactDiagram.reset();
	},
	
	selectAndFocusOn: function (uris) {
		// summary:
		//		Find, select and center on the artifact identified by the passed URI
		var ns = this.impactDiagram.get('nodesStore');
		this.impactDiagram.getSelection().clear();
		dojo.forEach(uris, function (artifactUri) {
			ns.fetchItemByIdentity({identity: artifactUri, onItem: dojo.hitch(this, function (item) {
				var node = this.impactDiagram.getGraphElementFromDataItem(item);
				this.impactDiagram.getSelection().add(item);
				node.setSelected(true);
				if (uris.length === 1) {
					this.impactDiagram.centerOnNode(node, {animate: true});
				}
			})});
		}, this);
	},

	//@param impactAnalysis - contains the data to show in the diagram, if null diagram will show local saved data
	loadData: function(impactAnalysis)
	{
		var ia;

		this.initDiagramsAndTree();
		ia = impactAnalysis ? impactAnalysis : this.editor.editorInput.impactAnalysis;

		if(ia.isReadyToRun()) {
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [messages.computeImpactAnalysis]);
			var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.computeImpactAnalysis(ia);
			deferred.addCallback(this, dojo.hitch(this, function(result) {
					if (!result) {
						return; // no execution result
					}

					// Translate html escape characters in the result node names:
					dojo.forEach(result.elementsData.items, function(item){
						item.name = jazz.util.html.unescape(item.name);
					});

					this.originalResult = dojo.clone(result);
				
					this.initImpactData(result);
					this.loadDiagram(result);
				})
			);
			deferred.addErrback(this, function(error){
				dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
				var xhr = deferred.ioArgs.xhr;
				if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
					var lqeInfo = dojo.fromJson(xhr.responseText);
					this.editor.showMessage({
						type: lqeInfo.type,
						summary: lqeInfo.summary,
						details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
						closable: true
					});
				} else {
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.computeImpactAnalysisError,
						errorObject: error,
						closable: true
					});
				}
			});
		}
	},

	reloadDiagramAndTree: function (changeDistance)
	{
		this.initDiagramsAndTree();
		var result = dojo.clone(this.originalResult);
		this.initImpactData(result, changeDistance);
		this.loadDiagram(result);
	},
	
	loadTree:function(result)
	{
		if(!this.treeSection) {
			return;
		}
		var treeCollapsibleSection = new rmps.ui.CollapsibleSection({
			primaryTitle: messages.panelTreeNavTitle,
			collapsible: false
		});
		dojo.style(treeCollapsibleSection.domNode, 'margin', '-3px');
		this.treeSection.appendChild(treeCollapsibleSection.domNode);
		var treeContent = treeCollapsibleSection.containerNode;
		dojo.style(treeContent, 'paddingLeft', '10px');
		dojo.style(treeContent, 'paddingTop', '5px');

		this.slider = new dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram._Slider({
			sresult: result ,
			distance: this.distance,
			diagram: this
		});
		treeContent.appendChild(this.slider.domNode);

		var viewTreeCaption =  dojo.doc.createElement("div");
		//var impactDef = this.editor.getImpactDefinition();
		viewTreeCaption.innerHTML = messages.navigationTreeLabel;
		treeContent.appendChild(viewTreeCaption);
		
		var treeW = new dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisTree({diagram: this});
		treeContent.appendChild(treeW.domNode);
		
		dojo.forEach(result.elementsData.items, function (artifact) {
			if (artifact.nodeof) {
				dojo.some(artifact.nodeof, function (node) {
					var found = false;
					if(node._reference === this.editor.editorInput.impactAnalysis.targetResourceUri.uri) {
						this.markSubgraph = artifact.URI;
						found = true;
					}
					return found;
				}, this);
			}
		}, this);
		this.addSubtree(treeW.getTree(), treeW.getTree(), result.treeData.items, this.markSubgraph ,result.treeData.numOfElements );
	},
	
	_checkForDiagramLimits: function (nArtifacts, nLinks) {
		var message,
			substitutions = {
				nArtifacts: nArtifacts,
				nLinks: nLinks,
				stopButtonLabel: messages.stop
			}; 
		if (dojo.isIE === 8 && (nArtifacts > 25 || nLinks > 25)) {
			message = messages.largeDiagramWarningIE8
		} else if (nArtifacts > 250 || nLinks > 250) {
			message = messages.largeDiagramWarning;
		}
		if (message) {
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
				summary: dojo.string.substitute(message, substitutions),
				closable: true
			});
		}
	},

	loadDiagram: function (result ) {
		// pick up the data sources...
		this.allElements = result.elementsData;
		var S = new dojo.data.ItemFileReadStore({data:result.elementsData});
		var SL = new dojo.data.ItemFileReadStore({data:result.relationsData});
		var ToolsS = new dojo.data.ItemFileReadStore({data:result.toolsData});
		
		this._checkForDiagramLimits(result.elementsData.items.length, result.relationsData.items.length);

		this.impactDiagram.set("linksStore", SL);
		this.impactDiagram.set("nodesStore", S);
		
		this.impactDiagram.set("nodeLayout", this._getCurrentLayout());

		// Asynchronous diagram load
		var loader = new dm.analysis.app.ui.internal.diagram.DiagramLoader(
				this.impactDiagram, 
				S, 
				SL);
		this.editor.enableIAStopButton(true);
				
		var iaLoadPromise = loader.createNodesAndLinks();
		var stopSubscription = dojo.subscribe('impactanalysis.stop', function () {
			iaLoadPromise.cancel();
		});
		iaLoadPromise.then(dojo.hitch(this, function () {
			this.loadTree(result);
			this.editor.enableIAStopButton(false);
			dojo.unsubscribe(stopSubscription);
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
		}), dojo.hitch(this, function(errResult) {
			this.editor.enableIAStopButton(false);			
			dojo.unsubscribe(stopSubscription);
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
			if(errResult.cancelled) {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.renderingCancelled,
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.computeImpactAnalysisError,
					errorObject: errResult,
					closable: true
				});
			}
		}), dojo.hitch(this, function(progressResult) {
			dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [progressResult.message]);
			console.info('Progress: ' + progressResult.message);
		}));
		
	},

	initDiagramsAndTree: function(){
		this.impactDiagram.reset();
		this.subgraphs = [];
		while(this.treeSection && this.treeSection.hasChildNodes()){
				this.treeSection.removeChild(this.treeSection.lastChild);
		}
		
		// Clean up any data that the popup menu may have created
		// for the nodes that were just deleted.
		this._popupMenu.cleanup();
	},

	filterDistance : function (result, distance) {
		var updatedItemList = [],
			updatedItemMap = {},
			updatedNodeofList;

		// include non-container items (those that are not logical dimensions)
		// that are within the specified distance
		dojo.forEach(result.elementsData.items, function (item) {
			if (!item.isLD && item.distance <= distance) {
				updatedItemList.push(item);
				updatedItemMap[item.URI] = item.URI;
			}
		});

		// include container items (i.e. logical dimensions) that
		// still reference items in the update map.
		dojo.forEach(result.elementsData.items, function (item) {
			updatedNodeofList = [];
			if (item.isLD == true) {
				dojo.forEach(item.nodeof, function (nodeof) {
					if (updatedItemMap[nodeof._reference]) {
						updatedNodeofList.push(nodeof);
					}
				});
				if (updatedNodeofList.length > 0) {
					item.nodeof = updatedNodeofList;
					updatedItemList.push(item);
				}
			}
		});

		result.elementsData.items = updatedItemList;
	},
	
	_getTargetUris: function (ia) {
		// summary:
		//		Return an array of URI's that are targets of the given impact analysis
		var targetUris = [ia.targetResourceUri];
		dojo.forEach(ia.addedElements, function (uri) { targetUris.push(uri); });
		return targetUris;
	},
	
	_calculateReachables: function (urisToRemove, previousLevelUris, reachableUris, linksField, linkEndField) {
		var nextLevelUris = [];
		dojo.forEach(previousLevelUris, function (knownUri) {
			var item = this.currentMapRes.itemMap[knownUri];
			dojo.forEach(item[linksField], function (link) {
				var uri = link[linkEndField];
				if (!reachableUris[uri] && dojo.indexOf(urisToRemove, uri) === -1) {
					nextLevelUris.push(uri);
					reachableUris[uri] = uri;
				}
			}, this);
		}, this);
		
		if (nextLevelUris.length > 0) {
			this._calculateReachables(urisToRemove, nextLevelUris, reachableUris, linksField, linkEndField);
		}
	},
	
	_removeFromArray: function (arr, keyField, key) {
		dojo.some(arr, function (entry, i) {
			var found = false;
			if (entry[keyField] === key) {
				arr.splice(i, 1);
				found = true;
			}
			return found;
		});
	},
	
	filterRemovedElements2: function (result, urisToRemove) {
		var reachableUris = {},
			targetUris,
			unreachables = [];
		
		// start with the target artifacts...	
		targetUris = this._getTargetUris(this.editor.editorInput.impactAnalysis);
		
		// make a 'map' of reachable uris
		dojo.forEach(targetUris, function (uri) {
			reachableUris[uri] = uri;
		});
		
		// don't let target artifacts be removed, not matter what...
		urisToRemove = dojo.filter(urisToRemove, function (uri) {
			return !reachableUris[uri];
		}, this);
		
		// figure out which other uri's we can reach, but up and down stream
		this._calculateReachables(urisToRemove, targetUris, reachableUris, "outLinks", "end");
		this._calculateReachables(urisToRemove, targetUris, reachableUris, "inLinks", "start");
		
		// calculate which ones are unreachable
		dojo.forEach(result.elementsData.items, function (artifact) {
			if(!reachableUris[artifact.URI]) {
				unreachables.push(artifact.URI);
			}
		});
		
		// remove the unreachables...
		dojo.forEach(unreachables, function (uri) {
			var item = this.currentMapRes.itemMap[uri],
				artifacts = result.elementsData.items,
				relations = result.relationsData.items;
			this._removeFromArray(artifacts, "URI", uri);
			dojo.forEach(item.inLinks, function (link) {
				this._removeFromArray(relations, "URI", link.URI);
			}, this);
			dojo.forEach(item.outLinks, function (link) {
				this._removeFromArray(relations, "URI", link.URI);
			}, this);
		}, this);
	},

	/**
	 * next - at first this is the targets + added elements.
	 */
	dfsScan : function (result /*result after remove */ ,current /*[]*/ , connectedMap /* - returned result {}*/)
	{
		var next = {},
			i,
			j;
		for(i = 0 ; i <current.length ; i ++)
		{
			
			var item = this.currentMapRes.itemMap[current[i]];
			connectedMap[current[i]] = current[i];
			var inlinks = item.inLinks;
			var outlinks = item.outLinks;
			for (j= 0 ; j < inlinks.length ; j++ )
			{
				if (connectedMap[inlinks[j].start] == null)//if not reached already
				{
					next[inlinks[j].start] = inlinks[j].start;
				}
			}
			for (j= 0 ; j < outlinks.length ; j++ )
			{
				if (connectedMap[outlinks[j].end] == null) //if not reached already
				{
					next[outlinks[j].end] = outlinks[j].end;
				}
			}
		}
		var nextArr = [];
		var key;
		for (key in  next)
		{
			nextArr.push(key);
		}
		if (nextArr.length > 0)
		{
			this.dfsScan(result ,nextArr, connectedMap);
		}
		
	},
	/*
	 * go over all the result elements and remove the elements that are not connceted and their links (in / out)
	 */
	removeDisconnectedElements: function(result, connectedMap /*{}*/) {
		var notConnected = [];
		result.elementsData.items = dojo.filter(result.elementsData.items, function (artifact) {
			var includeArtifact = true;
			if (!connectedMap[artifact.URI]) {
				notConnected.push(artifact.URI);
				includeArtifact = false;
			}
			return includeArtifact;
		});

		//remove the element's links that are disconnected
//		if (notConnected.length > 0)
//		{
//			//filterRemovedElements(result,notConnected);
//		}
	},
	
	calcMaxDistance: function(result)
	{
		var maxDistance = 0;
		dojo.forEach(result.elementsData.items, function (artifact) {
			if (artifact.isLD == false) {
				if (maxDistance < artifact.distance) {
					maxDistance = artifact.distance;
				}
			}
		}, this);
		this.distance = result.totalDistance = maxDistance;
	},
	
	_incrementMapCount: function (map, key) {
		map[key] = map[key] + 1 || 1;
	},
	
	_visitTreeData: function (rootItems, visitorMethod) {
		var inspectChildren;
		dojo.forEach(rootItems, function (treeItem) {
			inspectChildren = visitorMethod(treeItem);
			if(inspectChildren && treeItem.children) {
				this._visitTreeData(treeItem.children, visitorMethod);
			}
		}, this);
	},
	
	_visitTreeDataPostFix: function(rootItems, visitorMethod) {
		// summary:
		//		Traverse tree data in a post-fix fashion, visiting children before their parents.
		dojo.forEach(rootItems, function (treeItem, idx) {
			if(treeItem.children) {
				this._visitTreeDataPostFix(treeItem.children, visitorMethod);
			}
			visitorMethod(treeItem);
		}, this);
	},
	
	rebuildTheToolsLegendAndTree: function (result) {
		var artifactMap = [];
		
		var ldMap = {}; //{key:key val:key}
		var typesLDsMap = {}; //{key:key val:key}
		// update num of elements
		var isLDView = false;
		result.numOfElements = result.elementsData.items.length;
		result.treeData.numOfElements = result.elementsData.items.length	;
		
		// build maps
		dojo.forEach(result.elementsData.items, function (artifact) {
			artifactMap[artifact.URI] = artifact;
		});
		dojo.forEach(result.elementsData.items, function (artifact) {
			if (artifact.isLD == false && artifact.LDURI) {
				isLDView = true;
				this._incrementMapCount(ldMap, artifact.LDURI);
				this._incrementMapCount(typesLDsMap, artifact.LDURI + "_" + artifact.type);
			}
		}, this);

		
		if (isLDView) {
			//update ld tree
			var impacted = result.treeData.items[0];
			impacted.numOfElements = result.elementsData.items.length;
			var nonImpacted =  result.treeData.items[1];
			this.rebuildLDTreeElements(impacted,nonImpacted, typesLDsMap , ldMap);
		} else {
			//update structure tree with display counts.
			this._visitTreeDataPostFix(result.treeData.items, function (treeItem) {
				var numOfElements, artifact;
				if(treeItem.type === 'artifact') {
					artifact = artifactMap[treeItem.URI];
					treeItem.numOfElements =  artifact ? 1 : 0;
				} else {
					numOfElements = 0;
					dojo.forEach(treeItem.children, function (child) {
						numOfElements += child.numOfElements;
					});
					treeItem.numOfElements = numOfElements;
				}
			});
		}
	},
	
	
	rebuildLDTreeElements: function(treeElement ,nonImpactedList , typesLDsMap , ldMap )
	{
		var tmpChildren = [],
			i;
		for (i= 0; i < treeElement.children.length ; i ++ )
			{
				var tmpElement = treeElement.children[i];
				
				if (tmpElement.type == "element_type")
				{
					var counter = typesLDsMap[treeElement.URI + "_" + tmpElement.URI];
					if (counter  != undefined )
					{
						tmpElement.numOfElements = counter;
						tmpChildren.push(tmpElement);
					}
				}
				else // elt / e.g. system
				{
					var counter2 = ldMap[tmpElement.URI];
					if (counter2  != undefined)
					{
						tmpElement.numOfElements = counter2;
						tmpChildren.push(tmpElement);
					}
					else{
						var nonImpacted = {};
						nonImpacted.URI = tmpElement.URI;
						nonImpacted.name = tmpElement.name || '';
						nonImpacted.type = tmpElement.type;
						
						nonImpactedList.children.push(nonImpacted);
					} 
					if (tmpElement.children && tmpElement.children.length > 0)
					{
						this.rebuildLDTreeElements(tmpElement ,nonImpactedList , typesLDsMap , ldMap);
					}
				}
			}
			treeElement.children = tmpChildren;
			
	},
	
	rebuildTreeElementChildren: function (treeElement , typesMap)
	{
		var children = [];
		if (treeElement.children) {
			dojo.forEach(treeElement.children, function (treeNode) {
				var count = typesMap[treeElement.URI + "_" + treeNode.URI];
				if (count) {
					treeNode.numOfElements = count;
					children.push(treeNode);
				}
			}, this);
			treeElement.children = children;
		}
	},
	

	getElementindex: function(URI , arr){
		var i;
		for (i = 0 ; i < arr.length ; i ++ )
		{
			if (URI == arr[i].URI)
			{
				return i;
			}
		}
	},
	/*
	 * Item object
	 * item.uri , item.original (uri, name ,tool , node , type )
	 * item.inLink , item.outLink
	 *
	 * item.inlinks = map of Links object a-->thisObj , b-->thisObj
	 * item.outlinks = map of Links object thisObj-->c , thisObj-->d
	 *
	 * link object
	 * link.uri
	 * link.original (uri , source ,target)
	 *
	 */
	initImpactData : function (result , changeDistance)
	{
		// changeDistance argument is option. Default it to true if not specified
		if(changeDistance === undefined) {
			changeDistance = true;
		}
		if (result == null) {
			return;
		}
		
		this.currentMapRes = {};
		var itemMap = {};
		var linkMap = {};
		var nodesMap = {};
		this.currentMapRes.itemMap = itemMap;
		this.currentMapRes.linkMap = linkMap;
		this.currentMapRes.nodesMap = nodesMap;
		
		dojo.forEach(result.elementsData.items, function (artifact) {
			var item = dojo.clone(artifact);
			if (item.isLD == false) {
				item.inLinks = [];
				item.outLinks = [];
				itemMap[item.URI] = item;
			} else {
				nodesMap[item.URI] = item;
			}
		}, this);
		
		dojo.forEach(result.relationsData.items, function (link) {
			var clonedLink = itemMap[link.URI] = dojo.clone(link);
			// add links to items
			itemMap[link.start].outLinks.push(clonedLink);
			itemMap[link.end].inLinks.push(clonedLink);
		}, this);

		//handle the "removed elements"
		// remove elements
		if (this.editor.editorInput.impactAnalysis.removedElements.length > 0) {
			// rebuild only if removedElements > 0
			this.filterRemovedElements2(result, this.editor.editorInput.impactAnalysis.removedElements);
			// remove disconnected elements (from the target + added elements non-directed graph)
			var current = dojo.clone(this.editor.editorInput.impactAnalysis.addedElements);
			current.push(this.editor.editorInput.impactAnalysis.targetResourceUri);
			var connectedMap = {};
			this.dfsScan(result, current, connectedMap);
			this.removeDisconnectedElements(result, connectedMap);
		}
		if(changeDistance) {
			this.calcMaxDistance(result);
		}
		this.filterDistance(result, this.distance || result.totalDistance);
		this.rebuildTheToolsLegendAndTree(result);
	},

	removeElement: function (URI, name, type)
	{
		var ia = this.editor.editorInput.impactAnalysis;
		ia.removedElements.push(URI); // add the element
		this.editor.addRemovedElement(URI, name, type);
		//reload
		this.reloadDiagramAndTree(true);
	},

	setGridLayoutOnSubgraph :function (subgraph){
		subgraph.textShape.setFont({size: '10pt'});
		if (subgraph._data.item.URI ) {
			if (this.markSubgraph == subgraph._data.item.URI) {
				subgraph.setBorderColor('#FF0000');
			    subgraph._collapsedGroup.children[0].setStroke('#FF0000');
			}
		}
		this.subgraphs[this.subgraphs.length] = subgraph;
		subgraph.getGraph().setNodeLayout(common.selectLayout(common.LAYOUT_HIERARCHICAL));
        subgraph.getGraph().setAutomaticNodeLayout(false);
    },

	onNodeCreated : function(node){
		var d = node.getData(),
			isTargetNode,
			isFocalPointNode;

		// back link item to node...
		d.item.node = node;
		
		// apply color schema
		node.setBorderColor(colorTable.getElementBorderColor (d.item.color));
		node.setBackgroundColor(colorTable.getElementColor (d.item.color));
		
		// look for target nodes
		isTargetNode = (this.editor.editorInput.impactAnalysis.targetResourceUri === d.item.URI[0]);
		isFocalPointNode =  isTargetNode ||
			dojo.some( this.editor.editorInput.impactAnalysis.addedElements, function (added) {
				return added == d.item.URI[0];
			});
		
		if (isFocalPointNode) {
			node.setBorderStyle('longdash');
		}
		if (isTargetNode) {
			this._targetNode = node;
		}
    },

	onNodeClick : function(node) { 
		if (node) {
			if (this._popupMenu.isMenuShowingForElement(node)) {
				this._popupMenu.hidePopupMenu();
			} else {
				// If the poopup is showing for a different node, it
				// will automatically be closed and shown for this node
				this._popupMenu.showNodePopupMenu(node);
			}
		}
	},

	onFocusedElementChanged: function (oldNode, newNode) {
		if (newNode) {
			this.onNodeMouseEnter(newNode);
		}
	},

	onKeyPress: function (event) {
		// If a node has the focus
		if (this._focusNode) {
			var k = dojo.keys;
			if (event.keyCode === k.ENTER) {
				this._popupMenu.showNodePopupMenu(this._focusNode);
			}
		}
	},
	
	onNodeMouseEnter : function(node){
		// If another node had the focus, it does no more.
		if (this._focusNode) {
			this.onNodeMouseLeave(this._focusNode);
		}
		var d = node.getData();
		this.selectedItem = d;
		if (! node.isSelected())
		{
			node.setBorderColor(colorTable.getOnHoverColor());
			node.setBorderWidth(2);
		}
		
		// This node now has the focus
		this._focusNode = node;
	},

	onNodeMouseLeave : function(node){
		this.selectedItem =null;
		var d = node.getData();
		node.setBorderColor(colorTable.getElementBorderColor(d.item.color));
		var inAdded = false;
		var i;
		for (i = 0 ; i <  this.editor.editorInput.impactAnalysis.addedElements.length ; i++)
		{
			inAdded = this.editor.editorInput.impactAnalysis.addedElements[i] == d.item.URI;
			break;
		}
		if (this.editor.editorInput.impactAnalysis.targetResourceUri == d.item.URI || inAdded)
		{
			node.setBorderWidth(2);
		}else
		{
			node.setBorderWidth(1);
		}
			
		// Clear the node that had focus
		this._focusNode = null;
	},

	onLinkCreated: function(link,start,end)
	{
		try
		{
			link.setStrokeWidth("1");
			//link.setCurveTension(0.01); // not working OK
			if (link._data ) {
				if (link._data.item.intra == 'true') {
					link.setStrokeColor( colorTable.getLinkColor ("intra"));
				}
				else {
					link.setStrokeColor(colorTable.getLinkColor ("inter"));
				}
				if (link._data.item.crossLD == 'true') {
					link.setStrokeColor(colorTable.getLinkColor ("crossDomain"));
				}
				common.addDecoration(link, link._data.item.type, 0.5, ibm_ilog.diagram.LinkDecorationPositionType.Relative, ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center, ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle);
			}
			else {
				common.addDecoration(link, "contain", 0.5, ibm_ilog.diagram.LinkDecorationPositionType.Relative, ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center, ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle);
			}
		}
		catch (err) {
		}
	},

	// Method to insert nodes into the navigation tree
	addSubtree: function(root, tree, nodes, selected, numOfElements) {
		if (! nodes || ! tree) {
			return;
		}
		dojo.forEach(nodes, function (node) {
			if(node.numOfElements > 0) {
				var name = node.name || "";
				name = name.replace("-spa-" , " ");
				var nodeLabel;
				if (node.numOfElements && node.children ) {
					nodeLabel = name + " (" + node.numOfElements + ")";
				} else {
					nodeLabel = name ;
				}
				nodeLabel = dojox.html.entities.encode(nodeLabel);
				var item;
				if (node.type === 'artifact') {
					item = new jazz.ui.tree.SimpleItem({label: nodeLabel, uri: node.URI, nodeType: node.type, isSelected: false });
				} else {
					item = new jazz.ui.tree.IconItem({label:nodeLabel , iconClass: node.type , uri:node.URI , nodeType:node.type ,isSelected:false });
				}
				tree.addChild(item);
				this.addSubtree(root, item, node.children, selected,numOfElements);
			}
		}, this);
	},
	//moved
	performLayout: function() {
		setTimeout(dojo.hitch(this, this._performLayout()), 20);
	},
	
	_performLayout: function () {
		var i;
		this.impactDiagram.performGraphLayout();
		dojo.forEach(this.subgraphs, function (subgraph) {
			subgraph.setCollapsed(true);
		});

		if (this.editor.editorInput.impactAnalysis.removedElements.length > 0 && this.slider && this.slider.isFilterApplay() == true)
		{
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.removeElementsDistancefilterApplyed,
				closable: false
		});
		}
		else if (this.editor.editorInput.impactAnalysis.removedElements.length > 0){
			
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.removeElementsfilterApplyed,
				closable: false
			});
		}
		else if (this.slider && this.slider.isFilterApplay() == true){
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.distancefilterApplyed,
				closable: false
			});
		}
		else 
		{
			this.editor.hideMessage();
		}
		
		// ensure the primary focal point is in view
		this._focusOnTarget();
		
		// resolve images in nodes...
		// TODO: this can be more efficient if we could figure out a single request the return a sprite
		// and have every image source be a reference into that one sprte
		dojo.forEach(this.impactDiagram.getGraph().getNodes(), function (node) {
			var d = node.getData();
			if ( d.item.remoteServerUri[0]==null ) {
				
				if  (node.elementImg != null) {
					var queryObj = {
						icon: d.item.URI,
						serverIcon: 'nil'
					};
					var baseline = jazz.app.context.get().baseline;
					if(baseline) {
						dojo.mixin(queryObj, {"rmps.context": baseline});
					}
					var imgsrc = net.jazz.ajax._contextRoot + "/analysisinfo?" + dojo.objectToQuery(queryObj);
					node.elementImg.setShape({src:imgsrc}) ;
				}
				
			} else { 
				var remoteSrc = dojo.moduleUrl('dm.analysis.app', 'graphics/icons/remote1.gif');
				node.elementImg.setShape({src:remoteSrc}) ;
			}
		});
		
		dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram._Slider", [dijit._Widget, dijit._Templated], {

	diagram: null,
	slider:null,
	messages:messages,
	sresult:null,
	distance:null,
	widgetsInTemplate: true,
	sliderInput:null,
	templateString: '<div>' +
				'<div>${messages.setDistanceText}</div>'+
				'<table>' +
				'<tr>'+
				'<td style="width:70%"><div dojoAttachPoint="sliderDiv" ></Div></td>'+
				'<td style="width:30%"><label dojoAttachPoint="sliderInput" style="width:30px"></label></td>'+
				'</tr>'+
				'</table>' +
			'</div>',

	// init properties
	postCreate: function() {
		 this.slider = new dijit.form.HorizontalSlider({
			name: "slider",
			value: this.distance ? this.distance :this.sresult.totalDistance,
			minimum: 1,
			maximum: this.sresult.totalDistance,
			intermediateChanges: false,
			clickSelect: false,
            style: "width:155px",
			discreteValues:this.sresult.totalDistance
		 
		 });
		this.connect(this.slider, "onChange", function(value){ 
			if (value > 0) {
				this.diagram.distance = value;
				this.diagram.reloadDiagramAndTree(false); //mark that don't 
			}
		});

		this.sliderInput.innerHTML = this.distance ? this.distance +"/" + this.sresult.totalDistance:
		                                                     this.sresult.totalDistance + "/" + this.sresult.totalDistance ;
		this.sliderDiv.appendChild(this.slider.domNode);
	},
	startup: function(){

	},
	isFilterApplay: function() {
		return this.sresult.totalDistance != this.slider.value;
	}
});

}());


