(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.ImpactDefinitionsInlineEditor");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.NumberTextBox");

dojo.require("dm.common.app.ui.internal.TabManager");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");

dojo.require('jazz.util.html');

dojo.provide("dm.analysis.app.ui.internal.impactanalysis.panels.AddElementListPanel");
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.panels.RemoveElementListPanel");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages"); 

var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var TabManager = dm.common.app.ui.internal.TabManager;

dojo.declare(
	"dm.analysis.app.ui.internal.impactanalysis.ImpactDefinitionsInlineEditor", 
	[dijit._Widget, dijit._Templated], 
	{
		messages: messages,
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('dm.analysis.app','ui/internal/impactanalysis/templates/ImpactDefinitionsInlineEditor.html'),

		// init properties
		editor: null,

		constructor: function(args) {
			this.editor = args.editor;
		},
		
		postCreate: function() {
			this.inherited(arguments);

			var ia = this.editor.editorInput.impactAnalysis;
			this._loadData(ia);

			this._initializeTabs();

			// handle dirty awareness
			this.connect(this._distance,"onChange", dojo.hitch(this, function(newValue) {
				this.editor._updateDirty(parseInt(ia.iADepth, 10), newValue, this._distance);
			}));

			this.connect(this._impactType,"onchange", dojo.hitch(this, function() {
				this.editor._updateDirty(ia.iAType, this._impactType.value, this._impactType);
			}));

			this.subscribe('dm.analysis.app.ui.internal.impactanalysis.config.refresh', dojo.hitch(this, 'fillImpactTypesSelections'));

			var locateButton = this._elementToolbar.addButton(this, dojo.hitch(this.editor, '_locateElement'), {
				label: messages.iaEditorLocate,
				imgSrc: dojo.moduleUrl('dm.analysis.app', 'graphics/icons/search-icon.gif')
			});

			this._buttons = [locateButton];
		},

		_initializeTabs: function(){
			var tabStyleName= "Selected";

			this._tabManager = new TabManager();
			
			this._tabManager.registerTab(this._tabImpactDef, this._impactDefContent, tabStyleName);
			this._tabManager.registerTab(this._tabAdded, this._addedContent, tabStyleName);
			this._tabManager.registerTab(this._tabRemoved, this._removedContent, tabStyleName);

			this._addElementList = new dm.analysis.app.ui.internal.impactanalysis.panels.AddElementListPanel({editor: this.editor}, this._addElementList);
			this._removeElementList = new dm.analysis.app.ui.internal.impactanalysis.panels.RemoveElementListPanel({editor: this.editor}, this._removeElementList);

			dojo.addClass(this._addElementList.domNode, "element-list");
			dojo.addClass(this._removeElementList.domNode, "element-list");

			//TODO: Eventually if we decide to make a Tab and Tab Container Widget, the following two .startup() calls
			//		would be done via one call on the parent Tab Container to do .startup() on every Tab.
			//		...Or you can just be lazy and put it in their respective constructors or something.
			this._addElementList.startup();
			this._removeElementList.startup();

			this._tabManager.showTab(this._tabImpactDef); 
		},

		enableButtons: function(enable){
			var i,
				b;
			for (i = 0; i < this._buttons.length; i++)
			{
				b = this._buttons[i];
				if (b.setEnabled) {
					b.setEnabled(enable);
				}
			}
		},

		_loadData: function (impactAnalysis)
		{
			if (impactAnalysis.iADepth === null || impactAnalysis.iADepth === '' ) {
				this._distance.set('value', 15);
			} else {
				this._distance.set('value', (dojo.isString(impactAnalysis.iADepth) ? parseInt(impactAnalysis.iADepth, 10) : impactAnalysis.iADepth));
			}
			
			this.fillImpactTypesSelections();
		},
		
		addRemovedElement: function(uri, name, type)
		{
			this._removeElementList.addElement(uri, name, type, false);
		},

		getRemovedElementName: function(removedElement)
		{
			return this._removeElementList.getElementName(removedElement);
		},

		fillImpactTypesSelections : function()
		{
			while(this._impactType.hasChildNodes())
			{
				this._impactType.removeChild(this._impactType.lastChild);
			}
			
			var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getAllIAConfigs(this.editor.editorInput.projectId);
			deferred.addCallback(this, function(result) {
				var i,
					opt,
					config;
				for(i = 0 ; i < result.ImpactConfigsList.length ; i++) {
					config = result.ImpactConfigsList[i];
					opt = dojo.doc.createElement("option");
					opt.innerHTML = jazz.util.html.escape(config.name);
					opt.value = config.URI;
					if (this.editor.editorInput.impactAnalysis.iAType ===  config.URI)
					{
						opt.selected =true;
					} else if(this.editor.editorInput.impactAnalysis.iAType === null && config.systemDefaultConfig === "true") {
						opt.selected = true;
					}
					this._impactType.appendChild(opt);
				}
			});
			deferred.addErrback(this, function(error) {
				var xhr = deferred.ioArgs.xhr;
				if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
					var lqeInfo = dojo.fromJson(xhr.responseText);
					this.editor.showMessage({
						type: lqeInfo.type,
						summary: lqeInfo.summary,
						details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
						closable: true
					});
				} else {
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.errorGetAllIAConfigs,
						errorObject: error,
						closable: true
					});
				}
			});
		},
		
		getImpactDefinition: function() {
			var params = {
				iADepth: this._distance.get('value') ,
				iAType: this._impactType.value,
				iAViewType: 'no_ld'
			};

			return params;
		},

		reset: function(targetElementInfo){
			//set image and 
			while(this._locatedElement.hasChildNodes()){
				this._locatedElement.removeChild(this._locatedElement.lastChild);
			}
			var icon = dojo.doc.createElement("img");
			icon.src = targetElementInfo.iconUri;
			var resourceInfoHeader = dojo.doc.createElement("span");
			resourceInfoHeader.innerHTML =  jazz.util.html.escape(targetElementInfo.shortName);
			this._locatedElement.appendChild(icon);
			this._locatedElement.appendChild(resourceInfoHeader);
		}
});

}());
