/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| /com.ibm.xtools.rh.web.impact.app                                      |
 *|                                                                        |
 *| Copyright IBM Corporation 2011.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.application.MenuInitializer");

/** Initialize the Analysis menu by adding the Impact Analysis actions for
 *  browsing and creating impact analyses and impact analysis configurations.
 *  
 *  @author Paul Elder
 */

dojo.require("jazz.ui.menu.MenuItem");

dojo.require("dm.app.analysis.ui.internal.application.AnalysisActions");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.pages", "DMAnalysisMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.pages", "DMAnalysisMessages"); 

var AnalysisActions = dm.app.analysis.ui.internal.application.AnalysisActions;

/** Add the reports "browse" menu items */
dojox.lang.aspect.advise(dm.app.analysis.ui.internal.application.AnalysisMenu, "addAnalysisBrowseMenuItems", {
	after: function() {
		var menu = dojox.lang.aspect.getContext().instance;

		// If this call is for Reports menu items to be added
		if (menu.getCurrentSection() === AnalysisActions.SECTION_IMPACTANALYSIS) {
			
			menu.addItemByArgs({
				label: messages.allAnalysis,
				href: dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil.getAllAnalysisUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/impact.gif")
			});
			menu.addItemByArgs({
				label: messages.allAnalysisConfig,
				href: dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil.getAllConfigAnalysisUri(),
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_config.gif")
			});
		}
	}
});


/** Add the impact analysis "create" menu items */
dojox.lang.aspect.advise(dm.app.analysis.ui.internal.application.AnalysisMenu, "addAnalysisCreateMenuItems", {
	after: function() {
		var menu = dojox.lang.aspect.getContext().instance;
		
		// If this call is for report menu items to be added
		if (menu.getCurrentSection() === AnalysisActions.SECTION_IMPACTANALYSIS) {

			menu.add(new jazz.ui.menu.MenuItem({
				label: messages.createAnalysis,
				onClick: function(){
					dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil.createAnalysis();
				},
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/impact.gif")
			}));
			menu.add(new jazz.ui.menu.MenuItem({
				label: messages.createAnalysisConfig,
				onClick: function(){
					dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil.createAnalysisConfig();
				},
				icon: dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_config.gif")
			}));
		}
	}
});


})();