/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.config.AddFilterDialog");
dojo.require("jazz.ui.Dialog");

dojo.require("dm.common.app.ui.internal.rhlink.AnalysisConfigClient");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.StyledBox");
dojo.require("dojox.widget.Standby");

dojo.require('dijit.layout.ContentPane');
dojo.require('dijit.form.FilteringSelect');
dojo.require('dojo.data.ItemFileReadStore');

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.AddFilterDialog", null, {
	parent: null,
	iaConfig: null,
	projectId: null,
	filterType: "",

	domainCombo: null,
	addSelectionButton: null,
	typeSelected: null,
	standby: null,
	typeStore: null,
	_emptyListData: {
		identifier: 'key',
		label: 'value',
		items: []
	},

	dialog: null, //handle to the dialog
		
	constructor: function(parent, iaConfig, filterType) {
		this.parent = parent;
		this.iaConfig = iaConfig;
		this.filterType = dojo.isString(filterType) ? filterType : "";
		this.projectId = jazz.app.context.get().itemId;
	},
		
	createDialog: function(){
		var createTable = dojo.doc.createElement("table");

		//TODO: put up a radio button if the user wants to change between Link and Element

		//Domain:
		this.domainCombo = dojo.doc.createElement("select");
		this.createRelationRow(createTable, messages.iaEditorSelectionElmDialogTool, this.domainCombo);
		
		//A dojo data store for retrieved types
		this.typeStore = new dojo.data.ItemFileReadStore({clearOnClose: true});
		// provide some dummy data until we are ready to retrieve
		this.typeStore.data = {
			identifier: 'key',
			label: 'value',
			items: []
		};
		
		//Filtering Select:
		this.typeSelected = new dijit.form.FilteringSelect({
			id: 'typeSelected',
			store: this.typeStore,
			//name: 'key',
			searchAttr: 'value',
			style: 'width: 400px'
		});
		this.createRelationRow(createTable, messages.iaEditorSelectionElmDialogType, this.typeSelected.domNode);

		this.standby = new dojox.widget.Standby({target: this.typeSelected.domNode});
		
		//Add button
		this.addSelectionButton = dojo.doc.createElement("button");
		this.addSelectionButton.innerHTML = messages.iaEditorSelectionElmDialogAdd;
		dojo.connect(this.addSelectionButton,"onclick",this, function() { this.add(); });
		this.message =  dojo.doc.createElement("span");
		this.createRelationRow(createTable, null, this.addSelectionButton, this.message);
		
		//Fill Domain
		this.inprogress(true);
		this._getToolsTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getToolsTypes(this.projectId);
		this._getToolsTypesDeferred.addCallback(this, function(tools) {
			var noTools = true;
			dojo.forEach(tools.tools, dojo.hitch(this, function (item) {
				var optionChild = dojo.doc.createElement("option");
				optionChild.innerHTML = item.ToolType;
				optionChild.value = item.ToolURI;
				this.domainCombo.appendChild(optionChild);
				noTools = false;
			}));

			//Fill Filter Value
			this.loadTypeSelected();
		});
		this._getToolsTypesDeferred.addErrback(this, function(error) {
			var xhr = this._getToolsTypesDeferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.parent.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.parent.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetDomains,
					errorObject: error,
					closable: true
				});
			}
			this.dialog && this.dialog.close();
		});
		
		dojo.connect(this.domainCombo,"onchange",this, function() { 
			this.loadTypeSelected(); 
		});
		
		//Create Dialog:

		var dialogContent = new dijit.layout.ContentPane({
			id: 'addFilterDialogContents'
		});
		
		dialogContent.domNode.appendChild(createTable);
		
		dialogContent.domNode.appendChild(this.standby.domNode);
		
		this.dialog = new jazz.ui.Dialog({
			primaryTitle: messages.iaEditorSelectionElmDialogTitle + " (" 
				+ (this.filterType === this.parent.TYPE_ELEMENT ? messages.addFilterDialogResourceLabel : messages.addFilterDialogPredicateLabel)
				+ ")",
			modal: true,
			trim: jazz.ui.StyledBox.TrimStyles.BLUE_ALPHA_SHADOW,
			center: true,
			closeOnEsc: true,
			headerless: false,
			closable: true,
			width: 500 + "px",
			moveable: true,
			collapsible: false,
			showBackground: false,
			contentNode: dialogContent.domNode,
			destroyContents: true,
			onClose: this.parent.closeEdit
		});
		
		this.standby.startup();
	},
	
	loadTypeSelected : function()
	{
		this.inprogress(true);
		this.message.innerHTML = "";
		this.typeSelected.disabled = false;
		
		this.typeSelected.reset();

		if (this.filterType === this.parent.TYPE_ELEMENT) {
			this.fillSelectionWithElementTypes();
		} else {
			this.fillSelectionWithLinkTypes();
		}
	},
	
	fillSelectionWithLinkTypes: function()
	{
		// need to check if the type + tool does exist
		this._getSelectionssDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getTypesForTool(this.projectId , this.domainCombo.value , true /*since it is links type we want here*/);
		this._getSelectionssDeferred.addCallback(this, this._processTypeData);
		this._getSelectionssDeferred.addErrback(this, this.typeErrback);
	},
	
	fillSelectionWithElementTypes: function()
	{
		// need to check if the type + tool does exist
		this._getSelectionssDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getTypesForTool(this.projectId , this.domainCombo.value , false /*since it is links type we want here*/);
		this._getSelectionssDeferred.addCallback(this, this._processTypeData);
		this._getSelectionssDeferred.addErrback(this, this.typeErrback);
	},

	typeErrback: function(error)
	{
		var xhr = this._getSelectionssDeferred.ioArgs.xhr;
		if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
			var lqeInfo = dojo.fromJson(xhr.responseText);
			this.parent.editor.showMessage({
				type: lqeInfo.type,
				summary: lqeInfo.summary,
				details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
				closable: true
			});
		} else {
			this.parent.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: (this.filterType === this.parent.TYPE_ELEMENT) ? messages.errorGetElementTypes : messages.errorGetLinkTypes,
				errorObject: error,
				closable: true
			});
		}

		this.dialog.close();
	},
	
	_processTypeData: function (data) {
		var noData = data.types.length === 0, 
			alreadySelected = this.filterType === this.parent.TYPE_ELEMENT ? this.iaConfig.selectionElements : this.iaConfig.selectionLinks;

		// filter for already selected...
		data.types = dojo.filter(data.types, function (type) {
			return dojo.every(alreadySelected, function (s) {
				return type.key !== s.uri;
			});
		});
		
		// reload the typeStore data store
		this.typeStore.data = {
			identifier: 'key',
			label: 'value',
			items: dojo.map(data.types, function (type) {
				return { 
					key: type.key,
					value: type.value + ' (' + type.key + ')'
				};
			}, this)
		};
		this.typeStore.close();

		if (noData) { //disable other option
			this.typeSelected.disabled = true;
			this.message.innerHTML = messages.iaEditorSelectionElmDialogNoElmForToolMsg;
		}
		
		this.inprogress(false);
	},
	
	inprogress: function(inProgress){
		if (inProgress === true) {
			this.addSelectionButton.disabled = true;
			this.standby.show();
		} else {
			this.addSelectionButton.disabled = false;
			this.standby.hide();
		}
	},
	
	createRelationRow: function(table, name, value, message)
	{
		var tr = dojo.doc.createElement("tr");
		if (name != null)
		{
			var td1 = dojo.doc.createElement("td");
			td1.innerHTML = name;
			tr.appendChild(td1);
		}
		var td2 = dojo.doc.createElement("td");
		td2.appendChild(value);
		tr.appendChild(td2);
		
		if (message != null)
		{
			var td3 = dojo.doc.createElement("td");
			td3.appendChild(message);
			tr.appendChild(td3);
		}
		
		
		table.appendChild(tr);
	},
	
	
	add: function() {
		this.parent.updateAfterAddFilter(
			this.domainCombo.options[this.domainCombo.selectedIndex].text, 
			this.filterType,
			{ 
				domain: this.domainCombo.value,
				uri: this.typeSelected.item.key[0]
			}
		);

		this.dialog.close();
    }
	
});

}());