/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.config.AllIAConfigsEditor");

dojo.require("dijit._Widget");
dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.ui.editor.EditorUtil");
dojo.require("rmps.ui.Table");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient");
dojo.require("jazz.app.context");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysisConfig");
dojo.require('dm.analysis.app.ui.internal.impactanalysis.config.IAConfigInput');
dojo.require('rmps.ui.editor.PageHelper');

dojo.require('jazz.util.html');

dojo.require("jazz.ui.SimpleToolbar");
dojo.require("rmps.ui.util.ToolbarFocusUtil");
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

var UA_KEY = "com.ibm.xtools.impactanalysis.web.app.page_ImpactAnalysisConfigurations001";

var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.AllIAConfigsEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	isRefreshable: false,
	isReadOnly: true,	
	_diagramWidget: null,
	_content:null,
	editorInput:null,
	projectId:null,
	uaKey: UA_KEY,
	
	constructor: function (args) {
		this.editorInput= args.editorInput;
	},
	
	postCreate: function() {
		this.projectId = jazz.app.context.get().itemId;
		this.inherited(arguments);
		
	},
	
	createContentWidget: function() {		
		this._content = new dm.analysis.app.ui.internal.impactanalysis.config.AllIAConfigsEditor._Content({editor: this});
		return this._content;
	},

	//Cloned from BaseEditor; prevents domNode from being adjusted
	resize: function(size){
		if (!size) {
			return;
		}

		this._lastSize = size;

		var cbContainer = dojo.contentBox(this.containerNode);
		var mbContainer = dojo.marginBox(this.containerNode);
		var mbHeader = dojo.marginBox(this.headerContainerNode);
		
		var contentSize = {};
		if (size.h) {
			contentSize.h = size.h - mbContainer.h + cbContainer.h - mbHeader.h;
			this.containerNode.style.minHeight = contentSize.h + "px";
		}
		if (size.w) {
			contentSize.w = size.w - mbContainer.w + cbContainer.w;
			this.containerNode.style.minWidth = contentSize.w + "px";
		}

		if (this.contentWidget && this.contentWidget.resize) {
			this.contentWidget.resize(contentSize);
		}
	},

	addButtons: function(toolbar) {		
		toolbar.addButton(this, this.onRefreshClick, {
			label: messages.refresh,
			imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/refresh.gif")
		});
		toolbar.addSeparator(1);
	},
	
	addTextActions: function(){
		var self= this;
			self.addTextAction(messages.newConfig, 
			dojo.moduleUrl("dm.analysis.app", "graphics/icons/ia_config.gif"),
					dojo.hitch(self, self.onNewClick));
	},
	
	activate: function() {
		// until editing is supported, mark as dirty
	},
	
	onRefreshClick: function() {
		this._content.getAndPresentAllIAConfigs();
		
	},
	
	onNewClick: function() {
		dojo.hash(AnalysisUtil.getNewAnalysisConfigUri());
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.AllIAConfigsEditor._Content", [dijit._Widget, dijit._Templated], {
	
	messages: null, 
	
	widgetsInTemplate: true,
	templateString: '<div>' +
				'<div dojoAttachPoint="mainDiv">' +
				'</div>' +					
			'</div>',
	// init properties
	editor: null,
	_headers:null,
	_table:null,
	
	postCreate: function() {
		this._headers = this.createTableHeaders();
		this._table = new rmps.ui.Table();
		this.getAndPresentAllIAConfigs();
		this.subscribe("dm.analysis.app.ui.internal.impactanalysis.config.refresh", dojo.hitch(this, this.refreshTable));
		
	},
	
	refreshTable: function()
	{
		this.getAndPresentAllIAConfigs();
	},
	
	createTableHeader: function(id, args, label) {
		var header = {
				isOrderable: false
			};
		dojo.mixin(header, args);
		header.id = id;
		header.label = label;
		return header;
	},
	
	createTableHeaders: function() {  
		var headers = [
			this.createTableHeader("name", {filterParticipant: true, isOrderable: true} , messages.name),
			this.createTableHeader("description", {filterParticipant: true} ,  messages.description),
			this.createTableHeader("lastModified", { isOrderable: true, defaultAscending: false} ,  messages.lastModified),
			this.createTableHeader("modifier", {filterParticipant: true, isOrderable: false} ,  messages.modifier),
			this.createTableHeader("actions", {},  messages.actions)
			
		];
		return headers;
	},
	
	getAndPresentAllIAConfigs: function()
	{
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.getAllIAConfigs(this.editor.projectId);
		deferred.addCallback(this, function(result) {
			var tableData =[];
			dojo.forEach(result.ImpactConfigsList, function(entry) 
			{
				var resourceUri = entry.URI;
				var presentationUri = AnalysisUtil.getAnalysisConfigUri(entry.URI);
				
				var row = {
					link: presentationUri,
					linkTarget: "_self",
					resourceUri: resourceUri,
					iaConfig: entry,
					columns: []
				};
				
				dojo.forEach(this._headers, function(header) {
					var column = this.createColumnObject(entry, header.id, resourceUri, presentationUri);
					row.columns.push(column);
				}, this);
				tableData.push(row);
			
			}, this);
			
			this._table.setTableData(this._headers, tableData);
			var seconderyOrder = [];
			seconderyOrder.push({id:"lastModified" , isAscending: false});
			this._table.update(false,null,50,result.ImpactConfigsList.length,300,"name" , true , seconderyOrder);
			this.mainDiv.appendChild(this._table.domNode);
		});
		deferred.addErrback(this, function(error) {
			var xhr = deferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetAllIAConfigs,
					errorObject: error,
					closable: true
				});
			}
		});
	},
	
	createColumnObject: function(rowObj, columnId, resourceUri, presentationUri) {
		var result = {};
		if (columnId === "name") {
			result.data = rowObj.name;
			result.link = true;
		} else if (columnId === "description") {
			result.data = rowObj.description; 
			
		} else if (columnId === "lastModified") {
			result.data = rowObj.lastModified;
			result.style = { color: "#808080" };
			
		} else if (columnId === "modifier") {
			result.data = rowObj.modifier; 
		} else if (columnId === "actions") {
			result.init = dojo.hitch(this, this.createRowActions);
		}	
		return result;
	},
	
	
	createRowActions: function(rowObj, tableRow, rowIdx, columnConfig, cell, colIdx) {
		// Get the impact analysis configuration
		var iaConfig = rowObj.iaConfig;
		
	    // Create a toolbar to be the content of the table cell
		iaConfig._actionToolbar = new jazz.ui.SimpleToolbar();  // ToolbarFocusUtil requires a toolbar
		dojo.style(iaConfig._actionToolbar.domNode, 'minWidth','8em');
		dojo.addClass(cell, "actions");
		cell.appendChild(iaConfig._actionToolbar.domNode);
		
		// Add the "edit" action
		var button;
		button = iaConfig._actionToolbar.addButton(this, dojo.hitch(this, this._editIAConfig, iaConfig), 
					{imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/edit.gif"), label: messages.edit});
		dojo.style(button.domNode, "visibility", "hidden");
		
		// For system configurations, add the "reset" action and 
		// for user configurations, add a "delete" action.
		var buttonLabel = (iaConfig.systemDefined ? messages.reset : messages.deleteConfig);
		var buttonImage = (iaConfig.systemDefined ? "graphics/icons/refresh.gif" : "graphics/icons/remove.gif");
		button = iaConfig._actionToolbar.addButton(this, dojo.hitch(this, this._deleteIAConfig, iaConfig), 
					{imgSrc: dojo.moduleUrl("dm.analysis.app", buttonImage), label: buttonLabel});
		dojo.style(button.domNode, "visibility", "hidden");
		
		// Add listeners to show and hide the toolbar actions if necessary.
		iaConfig._actionToolbar.startup();
		ToolbarFocusUtil.addToolbarFocusListeners( { 
			actionToolbar: iaConfig._actionToolbar, 
			containerNode: tableRow, 
			actionClassName: ".jazz-ui-SimpleToolbar-button"
		});
	},
	
	_editIAConfig: function(iaConfig) {		
		var editActionUri = AnalysisUtil.getAnalysisConfigUri(iaConfig.URI);
		window.location.href = editActionUri;
	},
	
	_deleteIAConfig: function(iaConfig) {
		var substitutions = [jazz.util.html.escape(iaConfig.name)];
		var delConfirm = dojo.string.substitute((iaConfig.systemDefined ? messages.confirmReset : messages.confirmDelete), 
			substitutions);
		var delError = dojo.string.substitute((iaConfig.systemDefined ? messages.cantReset : messages.cantDelete), 
			substitutions);
		var delSuccess = dojo.string.substitute((iaConfig.systemDefined ? messages.successReset : messages.successDelete), 
			substitutions);
		if (! confirm (delConfirm))
		{
			return;
		}
		
		
		// The IA configuration summary has a "URI" property but the delete looks for a "uri" property.
		iaConfig.uri = iaConfig.URI;
		var deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.deleteIAConfig(iaConfig , false);
		deferred.addCallback(this, function(ia) {
			var iacInput,
				pageHelper;
			// close the editor, if it's open
			iacInput = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigInput({resourceUri: iaConfig.uri});
			pageHelper = rmps.ui.editor.PageHelper.getInstance().closeEditorInput(iacInput, true, true);
			rmps.ui.editor.PageHelper.updateDirtyStatus(pageHelper.pageId);
			
			// destroy the toolbar so that the listeners in ToolbarFocusUtil are removed.
			if (iaConfig._actionToolbar) {
				iaConfig._actionToolbar.destroy();
				delete iaConfig._actionToolbar;
			}
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: delSuccess,
				closable: true
			});
			
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.config.refresh");
			window.location.href = AnalysisUtil.getAllConfigAnalysisUri() + "&ts="+ new Date().getTime();
			
			
		});
		deferred.addErrback(this, function(error) {
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: delError,
				errorObject: error,
				closable: true
			});
		});
	}
	
	
});
}());
