/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.MultiSelect");

dojo.require("jazz.ui.Breadcrumbs");
dojo.require("rmps.core.error");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.UnlimitedNumberSpinner");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.config.CreateSelectionToolDialog");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.config.AddFilterDialog");

dojo.require("dm.common.app.ui.internal.TabManager");
dojo.require("dm.common.app.ui.internal.rhlink.AnalysisConfigClient");
dojo.require("dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil");

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis.config", "ImpactAnalysisConfigMessages"); 

var TabManager = dm.common.app.ui.internal.TabManager;
var AnalysisUtil = dm.analysis.app.ui.internal.impactanalysis.util.AnalysisUtil;

var PAGE_ID = dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor.PAGE_ID;

var DIRTY_GROUP_ID = "dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor.dirty";

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	isRefreshable: false,
	isReadOnly: false,	

	editorInput:null,
	content:null,
	//data
	_general:null,
	_tools:null,
	_selections:null,
	_advance:null,
	_breadCrumbs:null,
	dirtyTitle:null,
	_allTools:null,
	
	constructor: function (args) {
		this.editorInput= args.editorInput;
	},

	postCreate: function() {
		this._allTools = {};
		this.dirtyTitle = {};
		
		this._breadCrumbs= new jazz.ui.Breadcrumbs({segments: [{label: messages.iaConfigTitle, href: AnalysisUtil.getAllConfigAnalysisUri()}]});
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
		
		this.inherited(arguments);
		

	},

	activate: function() {
		// until editing is supported, mark as dirty
		//this.addDirty(this);
	},
	
	deactivate: function()
	{
	},
	
	startup: function() {
		this.content.startup();
	},
	
	isEditableTitle: function(){
		return !this.isReadOnly;
	},
	
	onTitleChange: function () {
		var oldValue = this.editorInput.iaConfig.shortName;
		this.editorInput.iaConfig.shortName = this.getTitleValue();
		this._updateDirty(oldValue, 
					this.getTitleValue(), 
					this.dirtyTitle);
	},
	
	createContentWidget: function() {		
		this.content = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor._Content({editor: this, isReadOnly: this.isReadOnly});
		return this.content;
	},
	
	addButtons: function(toolbar) {	
		toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.cancel,
			onClick: dojo.hitch(this, this.doCancel)
		}));
		
		this.inherited(arguments);
	},
	
	enable: function()
	{
		
	},
	disable: function()
	{
		
	},
	
	getEditableTitleNodeDetails: function () {
		// summary:
		//		Callback from BaseEditor providing default title for a new impact analysis
		return messages.newImpactAnalysisConfigTitle;
	},
	
	validate: function (/*Array*/errors) {
		// summary:
		//		Callback from BaseEditor to validate data prior to save
		if(this.editorInput.iaConfig.shortName === '') {
			errors.push(messages.errorMissingTitle);
		}
	},

	/** When Cancel is clicked in the analysis configuration editor, go
	*  back to the page for viewing all configurations.
	*/
	doCancel: function() {
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(PAGE_ID);
		pageHelper.closeEditor(this, true, true);
		window.location.href = AnalysisUtil.getAllConfigAnalysisUri() + "&ts="+ new Date().getTime();
	},
	
	doSave: function(taskMonitor) {
		this.disable();
		var iac = this.editorInput.iaConfig;
		//general
		iac.shortName  = this.getTitleValue();
		iac.description  = this._general.descriptionTextArea.get('value');
		iac.upStreamLevel  = this._general.upStreamLevelInput.getValue();
		iac.downStreamLevel  = this._general.downStreamLevelInput.getValue();
		//filters
		iac.enableDeselecting  = this._filters.enableDeselectingCheck.get('value') === "on";
		//advance
		iac.selectionInterLinks = [];
		dojo.forEach(this._advance.interLinks.options, function (option) {
			if (option.selected) {
				iac.selectionInterLinks.push(option.value);
			}
		}, this);
		
		//disable all buttons
		var deferred, isNew = false;
		if (iac.eTag) {
			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.updateIAConfig(iac);
		} else {
			deferred = dm.analysis.app.ui.internal.impactanalysis.models.RHImpactClient.createIAConfig(iac);
			isNew = true;
		}
		
		deferred.addCallback(this, function() {
			//when done , enable all button
			if (isNew) {
				// refresh the editor it with the now 'saved' editor input.
				this.editorInput.resourceUri = iac.uri;
				this.editorInput.removeState('id');
				this.refresh();
			} else {
				this.enable();
			}
			dojo.publish("dm.analysis.app.ui.internal.impactanalysis.config.refresh");
			
		});	
		
		deferred.addErrback(this, function(error) {
			this.enable();
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.errorSaving,
				errorObject: error,
				closable: true
			});
		});
		
		var task = taskMonitor.beginTask("saving", 1);
		task.handleDeferredWork(deferred, 1);
	},
	
	getResourceInfo: function(){
		return this.editorInput.resourceInfo;	
	},
	
	getProjectID: function(){
		return this.editorInput.projectId;	
	},
	
	_updateDirty: function(originalValue, newValue, dirtyKey) {
		var isEqual = originalValue === newValue;
		if (!isEqual && dojo.isArray(newValue) && originalValue && newValue && originalValue.length === newValue.length) {
			isEqual = true; // empty arrays
			isEqual = dojo.every(newValue, function(item, idx) {
				return item === originalValue[idx];
			});
		}
		if (isEqual) {
			this.removeDirty(dirtyKey, DIRTY_GROUP_ID);
		} else {
			this.addDirty(dirtyKey, DIRTY_GROUP_ID);
		}
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor._Content", [dijit._Widget, dijit._Templated], {
	messages: messages, 
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl('dm.analysis.app','ui/internal/impactanalysis/config/templates/IAConfigEditor.html'),
	
	// init properties
	editor: null,
	isReadOnly: false,
	
	postCreate: function() {
		var class_selectedTab= "Selected";
		this._tabManager = new TabManager();
		this._tabManager.registerTab(this._tabGeneral, this._tabGeneral_Content, class_selectedTab);
		this._tabManager.registerTab(this._tabFilters, this._tabFilters_Content, class_selectedTab);
		this._tabManager.registerTab(this._tabProject, this._tabProject_Content, class_selectedTab);
		this._tabManager.showTab(this._tabGeneral); 

		//get all tools map to be uses to present the tools name:
		this._getToolsTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getToolsTypes(this.editor.getProjectID());
		this._getToolsTypesDeferred.addCallback(this, function(result) {
			dojo.forEach(result.tools, function (tool) {
				this.editor._allTools[tool.ToolURI] = tool.ToolType;
			}, this);

			var sectionArgs = { editor: this.editor, isReadOnly: this.isReadOnly };

			this.editor._general = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_General(sectionArgs);
			this.editor._filters = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Filters(sectionArgs);
			this.editor._advance = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Advance(sectionArgs);

			this._tabGeneral_Content.appendChild(this.editor._general.domNode);
			this._tabFilters_Content.appendChild(this.editor._filters.domNode);
			this._tabProject_Content.appendChild(this.editor._advance.domNode);

			//TODO: Once the extent of reversable link functionality has been decided, Tools needs to be re-implemented somehow
			//this.editor._tools = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Tools(sectionArgs);
			//this._tabFilters_Content.appendChild(this.editor._tools.domNode);
		});
		this._getToolsTypesDeferred.addErrback(this, function(error) {
			var xhr = this._getToolsTypesDeferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetDomains,
					errorObject: error,
					closable: true
				});
			}

		});
		
	},
	
	startup: function(){
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_General", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<div>' +
			'<table style="overflow: scroll; width:100%; margin: 5px 0;" class="general" >' +
				'<tr> ' +
					'<td style="padding-right:10px; width:15%" dojoAttachPoint="_generalTitleCell"></td>' +
				'</tr>' +
				'<tr> ' +
					'<td style="padding-right:10px; width:15%" >${messages.iaEditorDesc}</td>' +
					'<td><textarea  class="instructions" dojoAttachPoint="descriptionTextArea"'+
							'dojoType="dijit.form.SimpleTextarea"'+
							'intermediateChanges="true"'+
							'cols="63" >'+
						'</textarea></td>'+
				'</tr>' +
				'<tr>' +
					'<td style="padding-right:10px; width:15%" >${messages.iaEditorUpStream}</td>' +
					'<td style="padding-right:10px; padding-left: 2px" ><input '+
						'dojoType="dm.analysis.app.ui.internal.impactanalysis.UnlimitedNumberSpinner"'+
						'dojoAttachPoint="upStreamLevelInput"/>'+
						'</td>' +
				'</tr>' +
				'<tr>' +
					'<td style="padding-right:10px; width:15%" >${messages.iaEditorDownStream}</td>' +
					'<td style="padding-right:10px; padding-left: 2px" ><input '+
						'dojoType="dm.analysis.app.ui.internal.impactanalysis.UnlimitedNumberSpinner"'+
						'dojoAttachPoint="downStreamLevelInput"/>'+
						'</td>' +
				'</tr>' +
			'</table>' +
		'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	iaConfig:null,
	
	postCreate: function() {
		this.iaConfig = this.editor.editorInput.iaConfig;
		
		this.inherited(arguments);
		
		if (this.iaConfig.systemDefined) {
			this._generalTitleCell.appendChild(dojo.doc.createTextNode(messages.iaEditorGeneralTitleSystemDefined));
		}
		
		this.descriptionTextArea.set('value',this.iaConfig.description);
		this.descriptionTextArea.set('disabled', this.isReadOnly);
		
		this.upStreamLevelInput.setValue(this.iaConfig.upStreamLevel);
		this.upStreamLevelInput.setDisabled(this.isReadOnly);
		
		this.downStreamLevelInput.setValue(this.iaConfig.downStreamLevel);
		this.downStreamLevelInput.setDisabled(this.isReadOnly);
		
		dojo.connect(this.shortNameTextBox,"onBlur", dojo.hitch(this, function() {
		//	this.editor.editorInput.setName(this.shortNameTextBox.get('value'));
			//this.editor.createTitleContents(true);
		}));
		////////////////////////////// handle changes if editor is not read only

		if (!this.isReadOnly) {
			dojo.connect(this.descriptionTextArea,"onChange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.description, 
								this.descriptionTextArea.get('value'), 
								this.descriptionTextArea);
				}));
				
			dojo.connect(this.upStreamLevelInput,"onchange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.upStreamLevel, 
								this.upStreamLevelInput.getValue(), 
								this.upStreamLevelInput);
				}));
				
			dojo.connect(this.downStreamLevelInput,"onchange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.downStreamLevel, 
								this.downStreamLevelInput.getValue(), 
								this.downStreamLevelInput);
				}));
		}
		
	},
	startup: function() {
		
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Filters", [dijit._Widget, dijit._Templated], {

	FILTER_TYPES: {
		ELEMENT: messages.iaEditorSelectionElement,
		LINK: messages.iaEditorSelectionLink
	},

	TYPE_LINK: "LINK",
	TYPE_ELEMENT: "ELEMENT",

	widgetsInTemplate: true,
	messages: messages,
	templateString: '<div>' +
			'<table style="overflow: scroll; width:100%" class="general" >' +
				'<tr>' +
					'<td style="width:1.5em" ><input type="checkbox" dojoAttachPoint="enableDeselectingCheck" dojoType="dijit.form.CheckBox"></td>' +
					'<td>${messages.iaEditorIgnoreSelected}</td>' +
				'</tr>' +
			'</table>' +
			'<div dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.iaEditorFilteredElements}" readOnly="false" collapsible="false" dojoAttachPoint="resourceTypeSection">' +
				'<div dojoAttachPoint="elementsTableDiv"/>' +
			'</div>' +
			'<div dojoType="rmps.ui.CollapsibleSection" primaryTitle="${messages.iaEditorFilteredLinks}" readOnly="false" collapsible="false" dojoAttachPoint="linkTypeSection"></div>' +
				'<div dojoAttachPoint="linksTableDiv"/>' +
			'</div>' +
		'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	iaConfig:null,
	
	postCreate: function() {
		this.iaConfig = this.editor.editorInput.iaConfig;	
		
		this.inherited(arguments);

		this.enableDeselectingCheck.set('checked' ,this.iaConfig.enableDeselecting );
		this.enableDeselectingCheck.set('disabled', this.isReadOnly);
		
		if (!this.isReadOnly) {
			dojo.connect(this.enableDeselectingCheck,"onChange", dojo.hitch(this, function() {
				this.editor._updateDirty(this.iaConfig.enableDeselecting, 
								this.enableDeselectingCheck.get('checked'), 
								this.enableDeselectingCheck);
				}));
		}

		//ELEMENTS:

		if (!this.isReadOnly) {
			// put the button in the section...
			this.resourceTypeSection.addImageAction(
				dojo.moduleUrl("dm.common.app", "graphics/icons/add.gif").uri,
				messages.iaEditorTypesAddToolTip,
				dojo.hitch(this, this._addFilter, this.TYPE_ELEMENT)
			);
		}
		
		this._elementsTable = new jazz.ui.StyledTable({
			columns: [
				{ label: messages.iaEditorElementType },
				{ label: messages.actions, className: "actions", width: "25%" }
			]
		});
		this._elementsTable.containerNode = this._elementsTable.tbody;
		dojo.style(this._elementsTable.domNode, 'width', '100%');
		this.elementsTableDiv.appendChild(this._elementsTable.domNode);
		this._elementsTable.startup();

		dojo.forEach(this.iaConfig.selectionElements, function(element) {
			this._addFilterRow(this.editor._allTools[element.domain], this.TYPE_ELEMENT,  element );
		}, this);


		//LINKS:

		if (!this.isReadOnly) {
			// put the button in the section...
			this.linkTypeSection.addImageAction(
				dojo.moduleUrl("dm.common.app", "graphics/icons/add.gif").uri,
				messages.iaEditorTypesAddToolTip,
				dojo.hitch(this, this._addFilter, this.TYPE_LINK)
			);
		}
		
		this._linksTable = new jazz.ui.StyledTable({
			columns: [
				{ label: messages.iaEditorLinkType },
				{ label: messages.actions, className: "actions", width: "25%"}
			]
		});
		this._linksTable.containerNode = this._linksTable.tbody;
		dojo.style(this._linksTable.domNode, 'width', '100%');
		this.linksTableDiv.appendChild(this._linksTable.domNode);
		this._linksTable.startup();	

		dojo.forEach(this.iaConfig.selectionLinks, function(link) {
			this._addFilterRow(this.editor._allTools[link.domain], this.TYPE_LINK,  link );
		}, this);
	},

	startup: function() {
		
	},
	
	/** Filter type can be LINK or ELEMENT */
	_addFilter: function(filterType) {
		new dm.analysis.app.ui.internal.impactanalysis.config.AddFilterDialog(this, this.iaConfig, filterType)
			.createDialog();
	},

	updateAfterAddFilter: function(domainText, filterType, filterValue) {
		if (filterType == this.TYPE_ELEMENT) {
			this.iaConfig.selectionElements.push(filterValue);
		} else {
			this.iaConfig.selectionLinks.push(filterValue);
		}
		
		this._addFilterRow(domainText, filterType, filterValue);
		this.editor.addDirty(this.selectionsSection, DIRTY_GROUP_ID);
	},
	
	_addFilterRow: function(domainText, filterType, filterValue) {
		var row = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_FilterRow({
			selectionsSection: this,
			domainText: domainText,
			filterType: filterType,
			filterValue: filterValue,
			isReadOnly: this.isReadOnly
		});

		if (filterType == this.TYPE_ELEMENT) {
			this._elementsTable.tbody.insertBefore(row.domNode, this._elementsTable.tbody.firstChild);
		} else {
			this._linksTable.tbody.insertBefore(row.domNode, this._linksTable.tbody.firstChild);
		}

		row.startup();
	},

	destroy: function() {
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_FilterRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<tr>' +
			/*'<td style="width: 25%;">' +
				'<div dojoAttachPoint="filterTypeNameDiv">' +
				'</div>' +
			'</td>' +*/
			'<td>' +
				'<div dojoAttachPoint="filterTypeUriDiv">' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 25%;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	selectionsSection: null,
	domainText: null,
	filterType: null,
	filterValue: null,
	isReadOnly: false,
	
	postCreate: function() {

		// Create the resource link
		dm.analysis.app.ui.internal.impactanalysis.util.ResourceLinkUtil.createResourceLink(this.filterTypeUriDiv, {
			uri: this.filterValue.uri,
			label: this.filterValue.uri
		});
		
		// add remove button
		if (!this.isReadOnly) {
			this.toolbar.addButton(this, function(e) {
				dojo.stopEvent(e);
				this.remove();
				this.destroyRecursive();
				}, 
				{
				label: "Remove Selection",
				imgSrc: dojo.moduleUrl("dm.analysis.app", "graphics/icons/remove_grey.gif")
			});
		}			
		
	},
	
	remove: function() {
		var table, collection;

		if (this.filterType === this.selectionsSection.TYPE_ELEMENT){
			table = this.selectionsSection._elementsTable;
			collection = this.selectionsSection.iaConfig.selectionElements;
		} else {
			table = this.selectionsSection._linksTable;
			collection = this.selectionsSection.iaConfig.selectionLinks;
		}

		// remove entry from table
		dojo.some(table.tbody.childNodes, function (node) {
			var found = false;
			if (node.id == this.domNode.id) {
				table.tbody.removeChild(node);
				found = true;
			}
			return found;
		}, this);
		
		// remove entry from IA config collection
		dojo.some(collection, function (item, i) {
			var found = false;
			if(item.uri == this.filterValue.uri) {
				collection.splice(i, 1);
				found = true;
			}
			return found;
		}, this);

		this.selectionsSection.editor.addDirty(this.selectionsSection, DIRTY_GROUP_ID);
		
	}
});
/*
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Tools", [dijit._Widget, dijit._Templated], {
		
	LINK_DIRECTION :	{TRUE:messages.iaEditorKeepDirection, 
						FALSE:messages.iaEditorReverseDirection},
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<div>' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" primaryTitle=${messages.iaEditorSelectedToolsTitle} isCollapsed="true" readOnly="false" collapsible="true"></div>' +
		'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	iaConfig:null,
	
	postCreate: function() {
		this.iaConfig = this.editor.editorInput.iaConfig;
		
		// create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: messages.iaEditorToolName, width: "50%"},
					{label: messages.iaEditorToolsLinkDirection ,width: "30%"},
					{label: "", className: "actions" , width: "20%"}
				]
		});
		this._table.containerNode = this._table.tbody;
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
		
		// create Add command
		if (!this.isReadOnly) {
			this._section.addAction(messages.iaEditorToolsAdd, messages.iaEditorToolsAddToolTip, dojo.hitch(this, function() {
			this._addTool();
			}));
		}
	
		// init
		for (var i = this.iaConfig.selectedTools.length - 1; i >= 0; --i) {
			this._addToolRow(this.iaConfig.selectedTools[i] , this.editor._allTools[this.iaConfig.selectedTools[i].tool]);
		}
		this._updateToolsCount();
	},
	
	_countTools: function() {
		return this.iaConfig.selectedTools.length;
	},
	
	_updateToolsCount: function() {
		var counter = this._countTools() ;
		this._section.setSecondaryTitle('(' + counter + ')');
		if (this._countTools() === 0) {
			this._section.collapse();
		} else
		{
			this._section.expand();
		}
	},
	
	// add the dialoge here
	_addTool: function(data) {
		var dialog = new dm.analysis.app.ui.internal.impactanalysis.config.CreateSelectionToolDialog(this , this.iaConfig);
	},
	updateAfterAddTool:function(toolData , toolName  ) {
		this.iaConfig.selectedTools[this.iaConfig.selectedTools.length] = toolData;
		this._addToolRow(toolData , toolName);
		this._updateToolsCount();
		this.editor.addDirty(this.toolsSection, DIRTY_GROUP_ID);
	},
	
	_addToolRow: function(data , toolName) {
		var row = new dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_ToolsRow({
			toolsSection: this,
			toolData: data,
			toolName: toolName,
			isReadOnly: this.isReadOnly
		});
		this._table.tbody.insertBefore(row.domNode, this._table.tbody.firstChild);
		row.startup();
		
		if (!data) {
			row.focus();
		}
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_ToolsRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<tr>' +
			'<td style="width: 50%;">' +
				'<div dojoAttachPoint="toolInput">' +
				'</div>' +
			'</td>' +
			'<td style="width: 30%;">' +
				'<select dojoAttachPoint="linkDirectionInput" >'+
			'</td>' +
			'<td class="actions" style="width: 25%;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	toolsSection: null,
	toolData: null, //{tool:"uri" , tooName:"name" , linksDirection:"true"}
	toolName: null,
	isReadOnly: false,
	
	postCreate: function() {

		this.toolInput.innerHTML = this.toolName;
		this.toolInput.focus();
		var optionChild = dojo.doc.createElement("option");
		optionChild.innerHTML = this.toolsSection.LINK_DIRECTION.TRUE;
		optionChild.value="TRUE";
		optionChild.selected = (this.toolData.linksDirection == "TRUE");
		this.linkDirectionInput.appendChild(optionChild);
		var optionChild1 = dojo.doc.createElement("option");
		optionChild1.innerHTML = this.toolsSection.LINK_DIRECTION.FALSE;
		optionChild1.value="FALSE";
		optionChild1.selected = (this.toolData.linksDirection == "FALSE");
		this.linkDirectionInput.appendChild(optionChild1);			
		this.linkDirectionInput.disabled = this.isReadOnly;
		
		if (!this.isReadOnly) {
			dojo.connect(this.linkDirectionInput,"onchange", dojo.hitch(this, function() {
				this.toolsSection.editor._updateDirty(this.toolData.linksDirection, 
									this.linkDirectionInput.value, 
									this.linkDirectionInput);
				this.toolData.linksDirection = this.linkDirectionInput.value;
			}));
		
		
			// add remove button
			this.toolbar.addButton(this, function(e) {
				dojo.stopEvent(e);
				this.remove();
				
				}, {
				label: messages.iaEditorToolsRemove,
				imgSrc: dojo.moduleUrl("dm.common.app", "graphics/icons/remove.gif")
			});
		}
	},
	
	focus: function() {
		this.toolInput.focus();
	},
	
	remove: function() { 
		//check if tool is in use
		for (var i = 0 ; i < this.toolsSection.iaConfig.selectionLinks.length ; i ++)
		{
			if (this.toolData.tool == this.toolsSection.iaConfig.selectionLinks[i].tool )
			{
				alert(messages.iaEditorToolUsedByLinkMsg);
				return;
			}
		}
		for (i = 0 ; i < this.toolsSection.iaConfig.selectionElements.length ; i ++)
		{
			if (this.toolData.tool == this.toolsSection.iaConfig.selectionElements[i].tool )
			{
				alert(messages.iaEditorToolUsedByElementMsg);
				return;
			}
		}
		
		jazz.util.array.removeByReference(this.toolsSection.iaConfig.selectedTools, this.toolData);
		var len = this.toolsSection._table.tbody.childNodes.length;
		var nodes = this.toolsSection._table.tbody.childNodes;
		for (i = 0; i < len; i++) {
			if (nodes[i].id == this.domNode.id) {
				this.toolsSection._table.tbody.removeChild(nodes[i]);
				break;
			}
		}
		this.toolsSection.editor.addDirty(this.toolsSection, DIRTY_GROUP_ID);
		this.toolsSection._updateToolsCount();
		this.destroyRecursive(); 
	}
});
*/
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.config.IAConfigEditor_Advance", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	messages: messages, 
	templateString: '<div>' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" readOnly="false" collapsible="true" isCollapsed="true">' +
					'<table style="overflow: scroll; width:100%" class="general" >' +
						'<tr> ' +
							'<td style="padding-right:10px" >${messages.iaEditorAdvancedInter}</td>' +
							'<td><select multiple="multiple" dojoAttachPoint="interLinks"  size="8"></td>'+
						'</tr>' +
				'</table>' +
			'</div>' +
		'</div>',
	
	// init properties
	editor: null,
	isReadOnly: false,
	iaConfig:null,
	
	postCreate: function() {
		this.iaConfig = this.editor.editorInput.iaConfig;
		
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(messages.iaEditorAdvancedTitle));
		
		this._section.setPrimaryTitle(titleNode);
		
		this.interLinks.disabled = this.isReadOnly;
		
		//handle dirty if editor is not read only
		if (!this.isReadOnly) {
			dojo.connect(this.interLinks,"onchange", dojo.hitch(this, function() {
				var tmpList = [];
				dojo.forEach(this.interLinks.options, function (option) {
					if (option.selected) {
						tmpList.push(option.value);
					}
				}, this);
				this._updateInterLinksCount(tmpList.length);
				this.editor._updateDirty(this.iaConfig.selectionInterLinks, 
									tmpList, 
									this.interLinks);
			}));	
		}
		
		
		var getLinkTypesDeferred = dm.common.app.ui.internal.rhlink.AnalysisConfigClient.getTypesForTool(this.editor.editorInput.projectId , null , true );
		getLinkTypesDeferred.addCallback(this, function(linkTypes) {
			var counter = 0;
			if (linkTypes.types.length > 0) {
				dojo.removeClass(this.editor.content._tabProject, "hidden");
			}
			dojo.forEach(linkTypes.types, function(item) {
				var optionChild = dojo.doc.createElement("option");
				optionChild.innerHTML = item.value;
				optionChild.value = item.key;
				if (this.checkIfInterLinkIsSelected(item.key))
				{
					optionChild.selected = true;
					counter++;
				}
				this.interLinks.appendChild(optionChild);
			}, this);
			this._updateInterLinksCount(counter);
		});
		getLinkTypesDeferred.addErrback(this, function(error) {
			var xhr = this._getSelectionssDeferred.ioArgs.xhr;
			if (xhr.getResponseHeader('X-lqe-web-oauth-url')) {
				var lqeInfo = dojo.fromJson(xhr.responseText);
				this.editor.showMessage({
					type: lqeInfo.type,
					summary: lqeInfo.summary,
					details: [{message: lqeInfo.userAction, status: lqeInfo.type}],
					closable: true
				});
			} else {
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.errorGetDomains,
					errorObject: error,
					closable: true
				});
			}
		});
		
	},
	
	
	_updateInterLinksCount: function(linkCount) {
		this._section.setSecondaryTitle('(' + linkCount + ')');
		if (this.linkCount === 0) {
			this._section.collapse();
		} else
		{
			this._section.expand();
		}
	},
	
	
	startup: function() {
		this.inherited(arguments);
		
		
	},
	
	checkIfInterLinkIsSelected: function(interLink)
	{
		return dojo.some(
			this.iaConfig.selectionInterLinks, 
			function (item) { return interLink === item; }, 
			this);
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
});

}());
