/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysis");
	
	dojo.require("dojox.data.dom");
	dojo.require('dojo.date.locale');

	dojo.require("rmps.core.resource.Resource");
	dojo.require("rmps.core.user.UserClient");
	dojo.require("jazz.util.html");

	///////////////////////////////////////////
	/* full XML
	<rdf:RDF xmlns:dcterms="http://purl.org/dc/terms/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rhImpactAnalysis="http://jazz.net/xmlns/rh/impacAnalysis/1.0/">
	<rhImpactAnalysis:ImpactAnalysis>
		<dcterms:description/>
		<rhImpactAnalysis:impactStatus>Draft</rhImpactAnalysis:impactStatus>
		<rhImpactAnalysis:owner/> --> todo add the creator user
		<rhImpactAnalysis:impactName>My Impact Analysis</rhImpactAnalysis:impactName>
		<rhImpactAnalysis:closeDate/>
		<rhImpactAnalysis:impactDueDate/>
		<rhImpactAnalysis:impactTimestamp/>
		<rhImpactAnalysis:startDate/>
		<rhImpactAnalysis:hasGolbalImpactConfig>
			<rhImpactAnalysis:ImpactConfigParam>
				<rhImpactAnalysis:configViewType>ByStructure</rhImpactAnalysis:configViewType>
				<rhImpactAnalysis:configType>DownStream</rhImpactAnalysis:configType>
				<rhImpactAnalysis:configDepth>4</rhImpactAnalysis:configDepth>
				<rhImpactAnalysis:configIsReduced>false</rhImpactAnalysis:configIsReduced>
			</rhImpactAnalysis:ImpactConfigParam>
		</rhImpactAnalysis:hasGolbalImpactConfig>
		<rhImpactAnalysis:hasArtifactImpact>
			<rhImpactAnalysis:ImpactArtifact>
				<rhImpactAnalysis:artifact rdf:resource="https://tp-sela.haifa.ibm.com:9444/rmps/models/1/1-4ba24ebb23ac10f4-O-8-000027c1"/>
				<rhImpactAnalysis:artifactStatus>MAIN</rhImpactAnalysis:artifactStatus> --> can also be ADD , REMOVE
				<rhImpactAnalysis:hasImpactConfig>
					<rhImpactAnalysis:ImpactConfigParam> -->place holder for future dev currently optional
						<rhImpactAnalysis:configViewType>BySystem</rhImpactAnalysis:configViewType>
						<rhImpactAnalysis:configType>UpStream</rhImpactAnalysis:configType>
						<rhImpactAnalysis:configDepth>2</rhImpactAnalysis:configDepth>
						<rhImpactAnalysis:configIsReduced>false</rhImpactAnalysis:configIsReduced>
					</rhImpactAnalysis:ImpactConfigParam>
				</rhImpactAnalysis:hasImpactConfig>
			</rhImpactAnalysis:ImpactArtifact>
		</rhImpactAnalysis:hasArtifactImpact>
	</rhImpactAnalysis:ImpactAnalysis>
</rdf:RDF>

*/
	
	//////////////////////////////////////////
		

	var UserClient = rmps.core.user.UserClient;

	var XMLNS = "http://open-services.net/ns/am#";
	var RH_IA  = "http://jazz.net/xmlns/rh/impacAnalysis/1.0/";
	var	DC    = "http://purl.org/dc/terms/";
	var	RDF   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	
	var IA_QUALIFIER ="rhImpactAnalysis";
	var CONTENT_TYPE_IA = "application/rdf+xml";
	var IA_STATUS = "impactStatus";
	var IA_NAME ="impactName";
	//config
	var IA ="ImpactAnalysis";
	var IA_TYPE ="configType";
	var IA_DEPTH ="configDepth";
	var IA_REDUCE ="configIsReduced";
	var IA_VIEW_TYPE ="configViewType";
	//excution
	var IA_ARTIFACT ="artifact";
	var IA_ARTIFACT_STATUS ="artifactStatus";
	
	var STATUS_DRAFT = "Draft";
	var STATUS_IN_PROGRESS = "InProgress";
	
	var ARTIFACT_MAIN = "MAIN";
	var ARTIFACT_ADD = "ADD";
	var ARTIFACT_REMOVE = "REMOVE";

	dojo.declare("dm.analysis.app.ui.internal.impactanalysis.models.ImpactAnalysis", rmps.core.resource.Resource, {

	// private properties
	description: '',
	shortName: '',
	eTag: null,

	////
	targetResourceUri: null,
	iAType: null,
	iAViewType: null,
	iADepth: 2, // default value
	iAReduceResult: null,
	impactStatus:null,

	addedElements:null,
	removedElements:null,
	
	//TBD add/Remove list / comments
	
	constructor: function() {
		this.resultUri = "";
		this.resultLastExe = "";
		this.resultCount = 0;
		this.addedElements = [];
		this.removedElements = [];
		
	},
	
	isReadyToRun: function () {
		// tests whether the Impact Analysis can be run.
		// Insists that targetResourceUri, and iAType are set
		return this.targetResourceUri !== null && this.iAType !== null;
	},
	
	getName: function()
	{
		return this.shortName;
	},
	
	getInprogress: function()
	{
		return this.STATUS_IN_PROGRESS;
	},
	
	getDraft: function()
	{
		return this.STATUS_IN_DRAFT;
	},
	
	
	getContributor: function() {
		var deferred;
		if (this._contributor && this._contributor.uri === this.contributor) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._contributor);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.contributor);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._contributor = user;
		}));
		return deferred;
	},

	getCreator: function() {
		var deferred;
		if (this._creator && this._creator.uri === this.creator) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._creator);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.creator);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._creator = user;
		}));
		return deferred;
	},
	
	/* returned XML ------------------------------------------------
	<?xml version="1.0" encoding="UTF-8"?> 
	<rdf:RDF xmlns:dcterms="http://purl.org/dc/terms/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rhImpactAnalysis="http://jazz.net/xmlns/rh/impacAnalysis/1.0/">
		<rhImpactAnalysis:ImpactAnalysis>
			<dcterms:description/>
			<rhImpactAnalysis:impactStatus>Draft</rhImpactAnalysis:impactStatus>
			<rhImpactAnalysis:owner/> --> todo add the creator user
			<rhImpactAnalysis:impactName>My Impact Analysis</rhImpactAnalysis:impactName>
			<rhImpactAnalysis:closeDate/>
			<rhImpactAnalysis:impactDueDate/>
			<rhImpactAnalysis:impactTimestamp/>
			<rhImpactAnalysis:startDate/>
			<rhImpactAnalysis:hasGolbalImpactConfig>
				<rhImpactAnalysis:ImpactConfigParam>
					<rhImpactAnalysis:configViewType>ByStructure</rhImpactAnalysis:configViewType>
					<rhImpactAnalysis:configType>DownStream</rhImpactAnalysis:configType>
					<rhImpactAnalysis:configDepth>4</rhImpactAnalysis:configDepth>
					<rhImpactAnalysis:configIsReduced>false</rhImpactAnalysis:configIsReduced>
				</rhImpactAnalysis:ImpactConfigParam>
			</rhImpactAnalysis:hasGolbalImpactConfig>
			<rhImpactAnalysis:hasArtifactImpact>
				<rhImpactAnalysis:ImpactArtifact>
					<rhImpactAnalysis:artifact rdf:resource="https://tp-sela.haifa.ibm.com:9444/rmps/models/1/1-4ba24ebb23ac10f4-O-8-000027c1"/>
					<rhImpactAnalysis:artifactStatus>MAIN</rhImpactAnalysis:artifactStatus> --> can also be ADD , REMOVE					
					</rhImpactAnalysis:hasImpactConfig>
				</rhImpactAnalysis:ImpactArtifact>
			</rhImpactAnalysis:hasArtifactImpact>
		</rhImpactAnalysis:ImpactAnalysis>
	</rdf:RDF>

	 */
	
	toXMLString: function() {
		var iaXML = '<?xml version="1.0" encoding="UTF-8"?> \n' ;
		iaXML+= '<rdf:RDF xmlns="' + XMLNS + 
		'"\n xmlns:dcterms="' + DC + 
		'"\n xmlns:rdf="'+ RDF +
		'"\n xmlns:' + IA_QUALIFIER + '="' + RH_IA + '"' +
		'>\n';
		iaXML+='<rhImpactAnalysis:ImpactAnalysis>';
		iaXML+= this.fragmentNode(IA_QUALIFIER ,IA_STATUS ,jazz.util.html.escape(this.impactStatus) , true  ) ;
		iaXML+= this.fragmentNode(IA_QUALIFIER ,IA_NAME ,jazz.util.html.escape(this.shortName) , true  ) ;
		iaXML+= '<dcterms:description>' + jazz.util.html.escape(this.description) + '</dcterms:description>\n';
		iaXML+= '<rhImpactAnalysis:owner/>\n'; //todo
		iaXML+= '<rhImpactAnalysis:startDate/>\n';
		iaXML+= '<rhImpactAnalysis:impactDueDate/>\n';
		iaXML+= '<rhImpactAnalysis:closeDate/>\n';
		//golbal config
		iaXML+= '<rhImpactAnalysis:hasGolbalImpactConfig>\n';
	    iaXML+= '<rhImpactAnalysis:ImpactConfigParam>\n';
		iaXML+= this.fragmentNode(IA_QUALIFIER ,IA_TYPE ,this.iAType , true  ) ;
		iaXML+= this.fragmentNode(IA_QUALIFIER ,IA_DEPTH ,this.iADepth , false  ) ;
		iaXML+= this.fragmentNode(IA_QUALIFIER ,IA_VIEW_TYPE ,this.iAViewType , true  ) ;
		iaXML+= this.fragmentNode(IA_QUALIFIER ,IA_REDUCE ,this.iAReduceResult , true  ) ;
		iaXML+= '</rhImpactAnalysis:ImpactConfigParam>\n';
		iaXML+= '</rhImpactAnalysis:hasGolbalImpactConfig>\n';
		//target artifact (main)
		iaXML+= this.fragmentArtifact(this.targetResourceUri ,ARTIFACT_MAIN );
		//added artifact
		var i;
		for (i = 0 ; i <  this.addedElements.length ; i ++)
		{
			iaXML+= this.fragmentArtifact(this.addedElements[i] ,ARTIFACT_ADD);
		}
		//removed element
        for (i = 0 ; i < this.removedElements.length ; i ++)
        {
            iaXML += this.fragmentArtifact(this.removedElements[i],ARTIFACT_REMOVE);
        }
		
		//END
		iaXML+='</rhImpactAnalysis:ImpactAnalysis>';
		iaXML += '</rdf:RDF>\n';
		return iaXML;
	},
	
	/*
	 * <rhImpactAnalysis:hasArtifactImpact>
				<rhImpactAnalysis:ImpactArtifact>
					<rhImpactAnalysis:artifact rdf:resource="https://tp-sela.haifa.ibm.com:9444/rmps/models/1/1-4ba24ebb23ac10f4-O-8-000027c1"/>
					<rhImpactAnalysis:artifactStatus>MAIN</rhImpactAnalysis:artifactStatus> --> can also be ADD , REMOVE					
				</rhImpactAnalysis:ImpactArtifact>
			</rhImpactAnalysis:hasArtifactImpact>
	 */
	fragmentArtifact: function (artifactTypeUri ,artifactType )
	{
		var iaXML = "<rhImpactAnalysis:hasArtifactImpact>\n <rhImpactAnalysis:ImpactArtifact>\n";
       iaXML += this.fragmentNode(IA_QUALIFIER ,IA_ARTIFACT_STATUS ,artifactType , true  ) ; 
       iaXML += this.fragmentResourceNode(IA_QUALIFIER ,IA_ARTIFACT ,artifactTypeUri ,true) ;
       iaXML += "</rhImpactAnalysis:ImpactArtifact> \n</rhImpactAnalysis:hasArtifactImpact>\n";
	   return iaXML;
	},
	
	fragmentNode: function(qn, property, value, not_encoded) {
		var adoptedValue= value? value.toString(): value;
		var xmlEncoded = not_encoded ?  value : dojox.html.entities.encode(adoptedValue);
		if (xmlEncoded === null)
		{
			return "<" + qn + ":"+ property +"></" + qn + ":"+ property +">\n";
		}
		else{
			return "<" + qn + ":"+ property +">" + xmlEncoded + "</" + qn + ":"+ property +">\n";
		}
		
	},
	
	fragmentResourceNode: function(qn, property, value, not_encoded) {
		var adoptedValue= value? value.toString(): value;
		var xmlEncoded = not_encoded ?  value : dojox.html.entities.encode(adoptedValue);
		if (xmlEncoded === null)
		{
			return '<' + qn + ':'+ property +' rdf:resource=""/>\n';
		}
		else{
			return '<' + qn + ':'+ property +' rdf:resource="' + xmlEncoded + '"/>\n';
		}
	},
	
	getEmptyImpactanalysis: function()
	{
		var date = dojo.date.locale.format(new Date(), {formatLength: "medium"});
		var iaXML = '<?xml version="1.0" encoding="UTF-8"?> <rdf:RDF\n'+
		'xmlns:dcterms="http://purl.org/dc/terms/"\n'+
		'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n'+
		'xmlns:rhImpactAnalysis="http://jazz.net/xmlns/rh/impacAnalysis/1.0/">\n'+
			'<rhImpactAnalysis:ImpactAnalysis>\n'+
			' <dcterms:description></dcterms:description> \n'+   
		    '<rhImpactAnalysis:impactStatus>Draft</rhImpactAnalysis:impactStatus>\n'+
		    '<rhImpactAnalysis:owner></rhImpactAnalysis:owner>\n'+
		    '<rhImpactAnalysis:impactName>My Impact Analysis-'+ date +'</rhImpactAnalysis:impactName>\n'+
		    '<rhImpactAnalysis:closeDate></rhImpactAnalysis:closeDate>\n'+
		    '<rhImpactAnalysis:impactDueDate></rhImpactAnalysis:impactDueDate> \n'+   
		    '<rhImpactAnalysis:startDate></rhImpactAnalysis:startDate>\n';
		 if (this.targetResourceUri)
		 {
			 iaXML+= this.fragmentArtifact(this.targetResourceUri ,ARTIFACT_MAIN );
		 } 
		iaXML +='</rhImpactAnalysis:ImpactAnalysis>\n';
		iaXML +='</rdf:RDF>\n';
		 
		return iaXML;
	},

	parse: function(iaNode, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}
		this.addedElements =[];
		this.removedElements =[];
		this.parseNodes(iaNode);
	},
	
	parseNodes: function(iaNode)
	{
		var i;
		for( i = 0 ; i < iaNode.childNodes.length; i++) {
		   var node = iaNode.childNodes[i];
		   if (node.nodeName === "rhImpactAnalysis:impactStatus") {
				if (node.childNodes.length > 0) 
				{
					this.impactStatus = node.childNodes[0].nodeValue;
				}
			}
			else if (node.nodeName === "rhImpactAnalysis:impactName") {
				if (node.childNodes.length > 0) 
				{
					this.shortName = node.childNodes[0].nodeValue;
				}
			}
			else if (node.nodeName === "rhImpactAnalysis:hasGolbalImpactConfig") {
				this.getGlobalConfig(node);
			}	
			else if (node.nodeName === "rhImpactAnalysis:hasArtifactImpact") {
				this.getArtifact(node);
			}
			else{
				if (node.childNodes) {
					this.parseNodes(node);
				}
			}		
		}
	},
	
	
	/*
	 * <rhImpactAnalysis:hasArtifactImpact>
				<rhImpactAnalysis:ImpactArtifact>
					<rhImpactAnalysis:artifact rdf:resource="https://tp-sela.haifa.ibm.com:9444/rmps/models/1/1-4ba24ebb23ac10f4-O-8-000027c1"/>
					<rhImpactAnalysis:artifactStatus>MAIN</rhImpactAnalysis:artifactStatus> --> can also be ADD , REMOVE					
				</rhImpactAnalysis:ImpactArtifact>
			</rhImpactAnalysis:hasArtifactImpact>
	 */
	getArtifact: function(iaNode)
	{
		var artifacttype;
		var artifactUri;
		
		var i;
		for( i = 0 ; i < iaNode.childNodes.length; i++) {
		   var node = iaNode.childNodes[i];
		   if (node.nodeName === "rhImpactAnalysis:artifact") {
				if (node.attributes.length > 0) 
				{
					artifactUri = node.attributes[0].nodeValue;
				}
			}
			else if (node.nodeName === "rhImpactAnalysis:artifactStatus") {
				if (node.childNodes.length > 0) 
				{
					artifacttype = node.childNodes[0].nodeValue;
				}
			}
			else
			{
				if (node.childNodes) {
					this.getArtifact(node);
				}
			}
		}
		
		if (artifacttype === ARTIFACT_ADD)
		{
			this.addedElements[this.addedElements.length] = artifactUri;
		}
		else if (artifacttype === ARTIFACT_MAIN)
		{
			this.targetResourceUri = artifactUri; 
		}
		else if (artifacttype === ARTIFACT_REMOVE)
		{
			this.removedElements[this.removedElements.length] = artifactUri;
		}
	},
	getGlobalConfig: function(iaNode)
	{
		 var i;
		 for (i = 0; i < iaNode.childNodes.length; i++) {
			 var node = iaNode.childNodes[i];

			 if (node.nodeName === "rhImpactAnalysis:configType") {
				 if (node.childNodes.length > 0) 
				 {
					 this.iAType = node.childNodes[0].nodeValue;
				 }
			 }
			 else if (node.nodeName === "rhImpactAnalysis:configDepth") {
				if (node.childNodes.length > 0) 
				{
					this.iADepth = node.childNodes[0].nodeValue;
				}
			 }
			 else if (node.nodeName === "rhImpactAnalysis:configViewType") {
				if (node.childNodes.length > 0) 
				{
					this.iAViewType = node.childNodes[0].nodeValue;
				}
			 }
			 else if (node.nodeName === "rhImpactAnalysis:configIsReduced") {
				 var val;
				 if (node.childNodes.length > 0) {
					 val = node.childNodes[0].nodeValue;
				 }
				 if (val) {
					 this.iAReduceResult = val === "true";
				 }
			}
			else{
				if (node.childNodes) {
					this.getGlobalConfig(node);
				}
			}	
			
		 }
	}
});

}());